// Main.cpp: Testet den Compiler, der Bytecode erzeugt, und den Bytecode-Interpreter.
// Eine 64-Bit-Gleitkommabibliothek wurde zum einen in ANSI C und zum anderen in dem C-Dialekt
// implementiert, den der Bytecode erzeugende Compiler akzeptiert. Die Funktionen der Gleitkommabibliothek
// werden mit Zufallszahlen aufgerufen, und es werden die Ergebnisse beider Implementierungen verglichen.
// Bei Abweichungen wird eine Fehlermeldung ausgegeben.

// Pointers can be created using the keyword 'pointer' and 'cpointer'. Pointers created using 'pointer' are
// only equivalent to ANSI C pointers if the memory the pointer points to really contains a value of the
// type given in the creation of the pointer. In this case, pointers created with 'pointer' are more space-efficient
// than pointers created with 'cpointer'. Pointers created using 'pointer' MUST NOT BE USED if they point
// to a memory position containing a type beeing different from the type given in the creation of the pointer
// (except for the zero pointer).

// Variablen, die mit local deklariert werden, entsprechen den C/C++ lokalen Variablen.
// Variablen, die mit var deklariert werden, entsprechen den C/C++ globalen Variablen, wenn sie auerhalb von
// einer Funktion deklariert werden, und den C/C++ statischen Variablen, wenn sie innerhalb einer Funktion
// deklariert werden.

#define MSCPP // Nur definieren, wenn unter Microsoft Visual Studio compiliert wird

#ifdef MSCPP
extern "C"
{
	#include "avr_f64.h"
}
#else
	#include "avr_f64.h"
#endif

#define cast_real_to_largest_uint(ValueX) ((ValueX).u64=f_float64_to_long((ValueX).u64))
#define cast_real_to_largest_sint(ValueX) ((ValueX).i64=f_float64_to_long((ValueX).u64))
#define cast_largest_sint_to_real(ValueX) ((ValueX).u64=f_long_to_float64((ValueX).i64))
#define cast_largest_uint_to_real(ValueX) ((ValueX).u64=f_long_to_float64((ValueX).u64))
#define real_add(x, y)              (x.u64=f_add((x.u64),(y.u64)))
#define real_sub(x, y)              (x.u64=f_sub((x.u64),(y.u64)))
#define real_mult(x, y)             (x.u64=f_mult((x.u64),(y.u64)))
#define real_div(x, y)              (x.u64=f_div((x.u64),(y.u64)))
#define real_modulo(x, y)           (x.u64=f_mod((x.u64),(y.u64), 0))
#define real_equal(x, y)            (f_compare((x.u64),(y.u64))==0)
#define real_not_equal(x, y)        (f_compare((x.u64),(y.u64))!=0)
#define real_less(x, y)             (f_compare((x.u64),(y.u64))<0)
#define real_greater(x, y)          (f_compare((x.u64),(y.u64))>0)
#define real_less_or_equal(x, y)    (f_compare((x.u64),(y.u64))<=0)
#define real_greater_or_equal       (f_compare((x.u64),(y.u64))>=0)
#define real_greater_or_equal(x, y) (f_compare((x.u64),(y.u64))>=0)
#define real_negate(x)              ((x).u64 ^= 0x8000000000000000ULL)

#include "BytecodeTargetedCompiler.h"

using BytecodeInterpreter::Value;
using BytecodeInterpreter::uint8;
using BytecodeInterpreter::uint16;
using BytecodeInterpreter::uint32;
using BytecodeInterpreter::uint64;
using BytecodeInterpreter::int8;
using BytecodeInterpreter::int16;
using BytecodeInterpreter::int32;
using BytecodeInterpreter::int64;


uint32 nibble_codesize=0;
uint8 *CodeOfProgram=0;

uint8 get_code_nibble(PROGRAM_COUNTER_TYPE pc)
{
#ifdef USE_ASSERT
	if(pc>=nibble_codesize)
		throw BytecodeInterpreter::RuntimeError("get_code_nibble Error", pc);
#endif
	return (pc&1) ? ((uint8)(CodeOfProgram[pc>>1]&15)) : ((uint8)((CodeOfProgram[pc>>1]>>4)&15));
}
uint8 get_code_byte(PROGRAM_COUNTER_TYPE pc)
{
#ifdef USE_ASSERT
	if(pc>=nibble_codesize)
		throw BytecodeInterpreter::RuntimeError("get_code_nibble Error", pc);
#endif
	//return (get_code_nibble(pc)<<4)+get_code_nibble(pc+1);
	if(0==(pc&1))
		return CodeOfProgram[pc>>1];
	else
		return (((uint8)(CodeOfProgram[pc>>1]&15))<<4) + ((uint8)((CodeOfProgram[(pc+1)>>1]>>4)&15));
}

void print_value(const Value& v)
{
	switch(v.type)
	{
		case Value::t_void:
			printf("void");	
			break;
		case Value::t_bool:
			printf("bool: %s", v.u8 ? "true" : "false");
			break;
		case Value::t_u8:
			printf("u8: %u  0x%x", v.u8, v.u8);
			break;
		case Value::t_i8:
			printf("i8: %d  0x%x", v.i8, v.i8);
			break;
		case Value::t_u16:
			printf("u16: %u  0x%x", v.u16, v.u16);
			break;
		case Value::t_i16:
			printf("i16: %d  0x%x", v.i16, v.i16);
			break;
		case Value::t_u32:
			printf("u32: %u  0x%x", v.u32, v.u32);
			break;
		case Value::t_i32:
			printf("i32: %d  0x%x", v.i32, v.i32);
			break;
		case Value::t_u64:
			printf("u64: 0x%.8x%.8x", (uint32)(v.u64>>32), (uint32)(v.u64&0xffffffff));
			break;
		case Value::t_i64:
			printf("i64: 0x%.8x%.8x", (uint32)(v.u64>>32), (uint32)(v.u64&0xffffffff));
			break;
		case Value::t_real:
			printf("r  : %.16g", v.r);
			break;
	}
	printf("      ");
}

uint64 bitwisedouble2uint64(double x)
{
	return *((uint64*)(&x));
}

uint64 f_from_double(double x)
{
	return *(uint64*)(&x);
}

double f_to_double(uint64 x)
{
	return *(double*)(&x);
}

float64 _f_sin(float64 x) { return f_sin(x); }
float64 _f_cos(float64 x) { return f_cos(x); }
float64 _f_tan(float64 x) { return f_tan(x); }
float64 _f_arcsin(float64 x) { return f_arcsin(x); }
float64 _f_arccos(float64 x) { return f_arccos(x); }
float64 _f_arctan(float64 x) { return f_arctan(x); }

namespace extern_function_names { enum
{	// C/C++ - Funktionen, die vom Programm, welches auf dem Bytecode-Interpreter luft, aufgerufen werden knnen
	// ("externe Funktionen").
	printsu, printu, printsr, prints, waitforkey,

	sqrt, exp, log, sin, cos, tan, arcsin, arccos, arctan,
	abs, cut_noninteger_fraction,

	float64_to_long, long_to_float64,
	compare, fmod
}; };

void extern_function_invoker(VARIABLE_INDEX_TYPE function_number, BytecodeInterpreter::BytecodeInterpreter& ip)
{	// Diese Funktion wird vom Bytecode-Interpreter aufgerufen, wenn das Programm eine externe Funktion aufruft.
	namespace nn=extern_function_names;
	switch(function_number)
	{
		case nn::waitforkey:
		{
			while(_getch()==0);
			break;
		}
		case nn::printu:
		{
			Value v;
			ip.pop_stack(&v);
			printf("\n------- ");
			print_value(v);
			printf("-------\n");
			break;
		}
		case nn::printsu:
		{
			Value v1, v2;
			ip.pop_stack(&v1);
			char *str=(char*)ip.var_addr_to_mem_addr(v1.u16);
			printf("\n");
			printf(str);
			ip.pop_stack(&v2);
			print_value(v2);
			printf("\n");
			break;
		}
		case nn::printsr:
		{
			Value v;
			ip.pop_stack(&v);
			char *str=(char*)ip.var_addr_to_mem_addr(v.u16);
			printf("\n");
			printf(str);

			ip.pop_stack(&v);
			double x=f_to_double(v.u64);
			printf("%.16g", x);
			printf("\n");
			break;
		}
		case nn::prints:
		{
			Value v;
			ip.pop_stack(&v);
			char *str=(char*)ip.var_addr_to_mem_addr(v.u16);
			printf("\n");
			printf(str);
			printf("\n");
			break;
		}
		case nn::sqrt:
		case nn::exp:
		case nn::log:
		case nn::sin:
		case nn::cos:
		case nn::tan:
		case nn::arcsin:
		case nn::arccos:
		case nn::arctan:
		case nn::abs:
		case nn::cut_noninteger_fraction:
		{
			typedef uint64 (*function_address_type)(uint64 float64_param);
			function_address_type funcs[]=
			{
				f_sqrt, f_exp, f_log, _f_sin, _f_cos, _f_tan, _f_arcsin, _f_arccos, _f_arctan,
				f_abs, f_cut_noninteger_fraction
				// Im enum ExternFunctionIDs mssen diese Funktionen genau in dieser Reihenfolge definiert sein, weil
				// die dem enum zugeordnete Zahl unten als Index fr dieses Array funcs[] verwendet wird.
			};
			Value v;
			ip.pop_stack(&v);
			ip.cast(&v, Value::t_u64);
			v.u64=funcs[function_number-nn::sqrt](v.u64); // Funktion aus der ANSI-C float64-Bibliothek aufrufen
			v.type=Value::t_u64;
			ip.push_stack(&v);
			break;
		}
		case nn::float64_to_long:
		{
			Value v;
			ip.pop_stack(&v);
			ip.cast(&v, Value::t_u64);
			v.i32=f_float64_to_long(v.u64);
			v.type=Value::t_i32;
			ip.push_stack(&v);
			break;
		}
		case nn::long_to_float64:
		{
			Value v;
			ip.pop_stack(&v);
			ip.cast(&v, Value::t_i32);
			v.u64=f_long_to_float64(v.i32);
			v.type=Value::t_u64;
			ip.push_stack(&v);
			break;
		}
		case nn::compare:
		{
			Value v1, v2;
			ip.pop_stack(&v1);
			ip.cast(&v1, Value::t_u64);
			ip.pop_stack(&v2);
			ip.cast(&v2, Value::t_u64);
			v1.i8=f_compare(v1.u64, v2.u64);
			v1.type=Value::t_i8;
			ip.push_stack(&v1);
			break;
		}
		case nn::fmod:
		{
			Value v1, v2, v3;
			ip.pop_stack(&v1);
			ip.cast(&v1, Value::t_u64);
			ip.pop_stack(&v2);
			ip.cast(&v1, Value::t_u64);
			ip.pop_stack(&v3);
			ip.cast(&v1, Value::VarIndexTypeNr);
			v1.u64=f_mod(v1.u64, v2.u64, (uint64*)ip.var_addr_to_mem_addr(v3.u16));
			v1.type=Value::t_u64;
			ip.push_stack(&v1);
			break;
		}
	}
}

#include "language_defines.h"

BEGIN_GLOBAL_SCOPE(TestFloat64Bibliothek)
// Ab hier beginnt das Programm (bzw. dessen Funktionen), welche(s) der Bytecode-Interpreter ausfhren soll.
// Die Syntax ist zwar hnlich, aber nicht identisch wie bei einem ANSI C Programm.

#define f_EULER_E ((float64_t)0x4005bf0a8b145769)	// 2.7182818284590452
#define f_NUMBER_PI ((float64_t)0x400921fb54442d18) // 3.1415926535897932
#define float64_NUMBER_ONE							((UINT64)0x3ff0000000000000ULL)	// 1.0
#define float64_NUMBER_PLUS_ZERO					((UINT64)0x0000000000000000ULL)	// 0.0
#define float64_ONE_POSSIBLE_NAN_REPRESENTATION		((UINT64)0x7fffffffffffffffULL)	// NaN
#define float64_PLUS_INFINITY						((UINT64)0x7ff0000000000000ULL)	// +INF
#define float64_MINUS_INFINITY						((UINT64)0xfff0000000000000ULL)	// -INF

	declare_void_function(Main, params());
	declare_void_function(TestFloat64, params());
	declare_void_function(f_split64, params(pointer(uint64), pointer(uint8), pointer(int16), pointer(uint64), uint8));
	declare_void_function(f_split_to_fixpoint, params(pointer(uint64), pointer(uint8), pointer(int16), pointer(uint64), int16));
	declare_void_function(f_combi_from_fixpoint, params(pointer(uint64), uint8, int16, pointer(uint64)));
	declare_function(uint64, f_add, params(uint64, uint64));
	declare_function(uint64, f_sub, params(uint64, uint64));
	declare_function(uint64, f_mult, params(uint64, uint64));
	declare_function(uint64, f_div, params(uint64, uint64));
	declare_function(pointer(uint8), copystr, params(pointer(uint8)));
	declare_void_function(strcpy, params(pointer(uint8), pointer(uint8)));
	declare_void_function(itoa10, params(int32, pointer(uint8)));
	declare_function(uint16, strlen, params(pointer(uint8)));
	declare_function(int32, atoi, params(pointer(uint8)));

	declare_extern_void_function(printsu, params(pointer(uint8), uint64));
	declare_extern_void_function(printu, params(uint64));
	declare_extern_void_function(waitforkey, params());
	declare_extern_void_function(printsr, params(pointer(uint8), uint64));
	declare_extern_void_function(prints, params(pointer(uint8)));

	declare_extern_function(uint64, sqrt, params(uint64));
	declare_extern_function(uint64, exp, params(uint64));
	declare_extern_function(uint64, log, params(uint64));
	declare_extern_function(uint64, sin, params(uint64));
	declare_extern_function(uint64, cos, params(uint64));
	declare_extern_function(uint64, tan, params(uint64));
	declare_extern_function(uint64, arcsin, params(uint64));
	declare_extern_function(uint64, arccos, params(uint64));
	declare_extern_function(uint64, arctan, params(uint64));
	declare_extern_function(uint64, abs, params(uint64));
	declare_extern_function(uint64, cut_noninteger_fraction, params(uint64));
	declare_extern_function(int32, float64_to_long, params(uint64));
	declare_extern_function(uint64, long_to_float64, params(int32));
	declare_extern_function(int8, compare, params(uint64, uint64));
	declare_extern_function(uint64, fmod, params(uint64, uint64, pointer(uint64)));

	declare_function(uint64, random, params());
	declare_function(uint64, frandom, params());
	declare_function(uint64, f_exp, params(uint64));
	declare_function(uint64, f_log, params(uint64));
	declare_function(uint64, f_abs, params(uint64));
	declare_function(uint64, f_cut_noninteger_fraction, params(uint64));
	declare_function(int32, f_float64_to_long, params(uint64));
	declare_function(uint64, f_long_to_float64, params(int32));
	declare_function(uint8, f_isnan, params(uint64));
	declare_function(uint8, f_finite, params(uint64));
	declare_function(int8, f_compare, params(uint64, uint64));
	declare_function(pointer(uint8), f_to_string, params(uint64, uint8, uint8));
	declare_function(uint64, f_strtod, params(pointer(uint8), pointer(pointer(uint8))));
	declare_function(uint64, f_eval_function_by_rational_approximation_fixpoint,
			params(uint64, uint8, uint8, pointer(uint64), uint8));
	declare_function(uint64, f_mod, params(uint64, uint64, pointer(uint64)));

	declare_function(pointer(uint8), f_to_decimalExp, params(uint64, uint8, uint8, pointer(int16)));
	declare_function(uint64, f_sqrt, params(uint64));
	declare_function(uint64, f_berechne_sinus_cosinus_tangens, params(uint64, uint8));
	declare_function(uint64, f_berechne_arcsin_arccos_arctan, params(uint64, uint8));
	declare_function(uint64, string_signature, params(pointer(uint8)));
	#define f_atof(str) (f_strtod((str), 0))
	#define f_sin(x) (f_berechne_sinus_cosinus_tangens((x), 0))	// Returns the sine of x.
	#define f_cos(x) (f_berechne_sinus_cosinus_tangens((x), 1))	// Returns the cosine of x.
	#define f_tan(x) (f_berechne_sinus_cosinus_tangens((x), 2))	// Returns the tangent of x.
	#define f_arcsin(x) (f_berechne_arcsin_arccos_arctan((x), 0))	// Returns the arc sine of x if -1<=x<=1 and NaN otherwise
	#define f_arccos(x) (f_berechne_arcsin_arccos_arctan((x), 1))	// Returns the arc cosine of x if -1<=x<=1 and NaN otherwise
	#define f_arctan(x) (f_berechne_arcsin_arccos_arctan((x), 2))	// Returns the arc tangent of x

	declare_void_function(reset_seed, params());

	var TemporaryMemory(array(uint8, 32)); // 32 Bytes im SRAM. Die Funktionen f_to_decimalExp() und f_to_string()
		// benutzen TemporaryMemory[] und drfen nur mit solchen Parametern aufgerufen werden, dass der erzeugte String
		// nicht lnger als 31 Zeichen (+ Nullbyte am Ende) ist.
	var rndseed_x(uint32, 1), rndseed_y(uint32, 2), rndseed_z(uint32, 4), rndseed_w(uint32, 8);

	declare_function(uint64, bitwise_real2uint64, params(real));
	declare_function(real, bitwise_uint64ToReal, params(uint64));

	void_function(Main)
		prints(copystr("123.987 + 7312.543 ="));
		prints(f_to_decimalExp(f_add(bitwisedouble2uint64(123.987), bitwisedouble2uint64(7312.543)), 20, 0, 0));
		prints(copystr("123.777 / 456.111 ="));
		prints(f_to_decimalExp(f_div(bitwisedouble2uint64(123.777), bitwisedouble2uint64(456.111)), 20, 0, 0));
		prints(copystr("sin(123.444)="));
		prints(f_to_decimalExp(f_sin(bitwisedouble2uint64(123.444)), 20, 0, 0));
		prints(f_to_decimalExp(sin(bitwisedouble2uint64(123.444)), 20, 0, 0));

		local dummy(uint64);

		var randdata(array(uint64, 100)); // !!! 800 Bytes im SRAM !!!
		for local i(uint8, 0); i<100; i++ do
			randdata[i]=(random() & 0x000fffffffffffffULL) | (((cast((cast((random()&0xffffffff), uint32)%10) + 1023, uint64)<<52)))
				| (random() & 0x8000000000000000ULL); // randdata[i] entspricht einer Zufallszahl zwischen -1024 und 1024
			//printsr(copystr("==> "), randdata[i]);
		end

		prints(copystr("\r\nStarte 100 Additionen mit Zufallszahlen ...\r\n"));
		for local i(uint8, 0); i<100; i++ do
			dummy=f_add(randdata[i], randdata[99-i]);
		end
		prints(copystr("100 Additionen fertig.\r\n\r\n"));

		prints(copystr("Starte 100 Divisionen mit Zufallszahlen ...\r\n"));
		for local i(uint8, 0); i<100; i++ do
			dummy=f_div(randdata[i], randdata[99-i]);
		end
		prints(copystr("100 Divisionen fertig.\r\n\r\n"));

		prints(copystr("Starte 50 Sinus-Werte von Zufallszahlen ...\r\n"));
		for local i(uint8, 0); i<50; i++ do
			dummy=f_sin(randdata[i]);
		end
		prints(copystr("50 Sinus-Werte fertig.\r\n\r\n"));

		prints(copystr("Teste 64 Bit Gleitkommabibliothek mit Zufallszahlen.\r\n\r\n"));

		TestFloat64();
	end

	// declare_function(uint64, bitwise_real2uint64, params(real));
	function(bitwise_real2uint64)
		param x(real);
		return(*(cast(&x, cpointer(uint64)))); // pointer(uint64) darf hier nicht verwendet werden, weil der Typ "verbogen" wird.
	end

	// declare_function(real, bitwise_uint64ToReal, params(uint64));
	function(bitwise_uint64ToReal)
		param x(uint64);
		return(*(cast(&x, cpointer(real)))); // pointer(real) darf hier nicht verwendet werden, weil der Typ "verbogen" wird.
	end

	function(copystr)
		param s(pointer(uint8));
		var mem(array(uint8, 128)); // !!! 128 Bytes Speicher im SRAM !!!
		local p(pointer(uint8), &mem[0]);
		while(0!=(*(p++)=*(s++))) end
		return(&mem[0]);
	end

	void_function(TestFloat64);
		local xr(real), yr(real), zr(real);
		local x(uint64), y(uint64), z1(uint64), z2(uint64);
		local errnr(uint8, 0);
		local i(uint32);
		local save_rndseed_x(uint64), save_rndseed_y(uint64), save_rndseed_z(uint64), save_rndseed_w(uint64);

		for i=0; i<0xffffffff; i++; do
			//if(0==(i%100))
				printsu(copystr("i = "), i);
			//end

			save_rndseed_x=rndseed_x;
			save_rndseed_y=rndseed_y;
			save_rndseed_z=rndseed_z;
			save_rndseed_w=rndseed_w;
			xr=bitwise_uint64ToReal(x=frandom()); yr=bitwise_uint64ToReal(y=frandom());

			prints(copystr("## 1"));
			if(xr+yr != bitwise_uint64ToReal(  f_add(x, y)  ))
				errnr=1; goto(error);
			end
			prints(copystr("## 2"));
			if(xr-yr != bitwise_uint64ToReal(  f_sub(x, y)  ))
				errnr=2; goto(error);
			end
			prints(copystr("## 3"));
			if(xr*yr != bitwise_uint64ToReal(  f_mult(x, y)  ))
				errnr=3; goto(error);
			end
			prints(copystr("## 4"));
			if(xr/yr != bitwise_uint64ToReal(  f_div(x, y)  ))
				errnr=4; goto(error);
			end
			prints(copystr("## 5"));
			if(sqrt(x) != f_sqrt(x))
				errnr=5; goto(error);
			end
			prints(copystr("## 6"));
			if(exp(x) != f_exp(x))
				errnr=5; goto(error);
			end
			prints(copystr("## 7"));
			if(log(x) != f_log(x))
				errnr=5; goto(error);
			end
			prints(copystr("## 8"));
			if(sin(x) != f_sin(x))
				errnr=5; goto(error);
			end
			prints(copystr("## 9"));
			if(cos(x) != f_cos(x))
				errnr=6; goto(error);
			end
			prints(copystr("## 10"));
			if(tan(x) != f_tan(x))
				errnr=7; goto(error);
			end

			prints(copystr("## 11"));
			if(arcsin(x) != f_arcsin(x))
				errnr=8; goto(error);
			end
			prints(copystr("## 12"));
			if(arccos(x) != f_arccos(x))
				errnr=9; goto(error);
			end
			prints(copystr("## 13"));
			if(arctan(x) != f_arctan(x))
				errnr=10; goto(error);
			end
			prints(copystr("## 14"));
			if(abs(x) != f_abs(x))
				errnr=10; goto(error);
			end
			prints(copystr("## 15"));
			if(cut_noninteger_fraction(x) != f_cut_noninteger_fraction(x))
				errnr=10; goto(error);
			end
			prints(copystr("## 16"));
			if(float64_to_long(x) != f_float64_to_long(x))
				errnr=11; goto(error);
			end
			prints(copystr("## 17"));
			if(compare(x, y) != f_compare(x, y))
				errnr=12; goto(error);
			end
			prints(copystr("## 18"));
			if(fmod(x, y, &z1) != f_mod(x, y, &z2) || z1!=z2)
				errnr=13; goto(error);
			end
			local q(int32);
			prints(copystr("## 19"));
			if(long_to_float64(q) != f_long_to_float64(q))
				errnr=14; goto(error);
			end
		end
		prints(copystr("------------FERTIG-------------------"));
		waitforkey();
		return();

		label(error);
		prints(copystr("*********** FEHLER *************"));
		printsu(copystr("i = "), i);
		printsu(copystr("errnr = "), errnr);
		printsu(copystr("rndseed_x = "), save_rndseed_x);
		printsu(copystr("rndseed_y = "), save_rndseed_y);
		printsu(copystr("rndseed_z = "), save_rndseed_z);
		printsu(copystr("rndseed_w = "), save_rndseed_w);
		while(1) end // *** Endlosschleife ***
	end

	function(frandom)
		local ex(int16), m(uint64, random());
		if((random()&127)==0)
			return(float64_NUMBER_PLUS_ZERO);
		end
		while((ex=random()&15)>=14) end
		if(random()&1)
			ex=1023+((random()&2047)>>ex);
		else
			ex=1023-((random()&2047)>>ex);
		end
		return(((random()&1)<<63) | ((cast(ex, uint64)&2047)<<52) | (m & 0xfffffffffffffULL));
	end

	void_function(reset_seed)
		rndseed_x=1; rndseed_y=2; rndseed_z=4; rndseed_w=8;
	end

	function(random)
		// KISS random generator
		local r(uint64);
		rndseed_x = 69069 * rndseed_x + 1327217885;
		rndseed_y ^= rndseed_y << 13;
		rndseed_y ^= rndseed_y >> 17;
		rndseed_y ^= rndseed_y << 5;
		rndseed_z = 18000*(rndseed_z&65535) + (rndseed_z>>16);
		rndseed_w = 30903*(rndseed_w&65535) + (rndseed_w>>16);
		r=cast(rndseed_x+rndseed_y+(rndseed_z<<16)+rndseed_w, uint64)<<32;
		rndseed_x = 69069 * rndseed_x + 1327217885;
		rndseed_y ^= rndseed_y << 13;
		rndseed_y ^= rndseed_y >> 17;
		rndseed_y ^= rndseed_y << 5;
		rndseed_z = 18000*(rndseed_z&65535) + (rndseed_z>>16);
		rndseed_w = 30903*(rndseed_w&65535) + (rndseed_w>>16);
		r |= (rndseed_x+rndseed_y+(rndseed_z<<16)+rndseed_w);
		return(r);
		   /*You may replace the two constants 18000 and 30903 by any
		   pair of distinct constants from this list:	   18000 18030 18273 18513 18879 19074 19098 19164 19215 19584 19599 19950 20088 20508 20544 20664 20814 20970 21153 21243
		   21423 21723 21954 22125 22188 22293 22860 22938 22965 22974 23109 23124 23163 23208 23508 23520 23553 23658 23865 24114
		   24219 24660 24699 24864 24948 25023 25308 25443 26004 26088 26154 26550 26679 26838 27183 27258 27753 27795 27810 27834
		   27960 28320 28380 28689 28710 28794 28854 28959 28980 29013 29379 29889 30135 30345 30459 30714 30903 30963 31059 31083
		   (or any other 16-bit constants k for which both k*2^16-1 and k*2^15-1 are prime)*/
	end

	void_function(strcpy)
		param dest(pointer(uint8)), src(pointer(uint8));
		while(0!=(*(dest++)=*(src++))) end
	end

	function(strlen)
		param s(pointer(uint8));
		local len(uint16, 0);
		while(*(s++))
			++len;
		end
		return(len);
	end

	function(atoi)
		param s(pointer(uint8));
		local n(int32), neg(bool, 0);
		if('-'==*s)
			neg=TRUE; ++s;
		end
		while(*s)
			n=n*10+(*(s++)-'0');
		end
		if(neg)
			n=-n;
		end
		return(n);
	end

	void_function(itoa10)
		param n(int32), strmem(pointer(uint8));
		if(n<0)
			*(strmem++)='-';
			n=-n;
		end
		local p(pointer(uint8), strmem), c(uint8);
		do_begin
			*(p++)='0' + (n%(UINT8)10);
			n/=(UINT8)10;
		end_while(n);
		*p=0; // nicht p++ !
		do_begin
			c=*(--p);
			*p=*strmem;
			*strmem=c;
		end_while(p!=strmem && p!=++strmem);
	end

	void_function(f_split64)
		param x(pointer(uint64)), f_sign(pointer(uint8)), f_ex(pointer(int16)), frac(pointer(uint64)), lshift(uint8);
		*frac = (*x) & (0xfffffffffffffULL);
		if(0==(*f_ex  =  (((*x)>>52) & 2047)))
			*frac=0;
		else if(2047!=(*f_ex))
			*frac |= 0x10000000000000ULL;
		end end
		*frac <<= lshift;
		*f_sign = ((*x)>>63) & 1;
	end

	void_function(f_split_to_fixpoint)
		param x(pointer(uint64)), f_sign(pointer(uint8)), f_ex(pointer(int16)), frac(pointer(uint64)), point_bitnr(int16);
		f_split64(x, f_sign, f_ex, frac, 0);
		if(0!=(*f_ex) && 2047!=(*f_ex))
			point_bitnr=(1023+52)-(*f_ex)-point_bitnr;
			if(point_bitnr<=-64 || point_bitnr>=64)
				*frac=0;
			else if(point_bitnr<0)
				*frac<<=-point_bitnr;
			else
				*frac>>=point_bitnr;
			end end
		end
	end

	void_function(f_combi_from_fixpoint)
		param x(pointer(uint64)), f_sign(uint8), f_ex(int16), frac2(pointer(uint64));
		local frac(pointer(uint64), frac2);
		*frac=*frac;
		local round(uint8, 0), w(uint64, *frac);
		if(0!=w)
			while(0==(w & 0xffffe00000000000ULL))
				w<<=8;
				f_ex-=8;
			end
			while(0==(w & 0xfff0000000000000ULL))
				w<<=1;
				--f_ex;
			end
			while(0!=(w & 0xff00000000000000ULL))
				round = 0!=(w&(1<<3));
				w>>=4;
				f_ex+=4;
			end
			while(0!=(w & 0xffe0000000000000ULL))
				round = 0!=(w&1);
				w>>=1;
				++f_ex;
			end
			if(round)
				++w;
				if(0!=(w & 0xffe0000000000000ULL))
					w>>=1;
					++f_ex;
				end
			end
			if(f_ex<=0) // Falls f_ex==0: Es werden keine unnormalisierten Zahlen auer Null untersttzt
				{ f_ex=0; w=0; } // +0 oder -0
			end
		else if(f_ex<2047)
			f_ex=0;
		end end
		if(f_ex>=2047)
			{ f_ex=2047; w=0; } // +INF oder -INF
		end
		*cast(x, pointer(uint64))=(cast(f_sign, uint64)<<63) | (cast(f_ex, uint64)<<52) | (w & 0xfffffffffffffULL);
	end

	declare_function(int8, f_shift_left_until_bit63_set, params(pointer(uint64)));
	function(f_shift_left_until_bit63_set)
		// Falls *w=0 ist oder falls das Bit mit der Nummer 63 nicht durch
		// mehrmaliges Linksschieben von *w gesetzt werden kann, wird *w=0 gesetzt und 64 zurckgeliefert.
		// Ansonsten wird *w so oft nach links geshiftet, bis das Bit mit der Nummer 63
		// gesetzt ist. Die Anzahl der erfolgten Links-Shifts wird zurckgeliefert.
		param w(pointer(uint64));
		local count(int8, 0);
		local mask(uint64);
		for mask=((UINT64)255LU)<<(63-7); 0==(mask & (*w)) && count<64; count+=8; do
			(*w) <<= 8;
		end
		for mask=((UINT64)1LU)<<63; 0==(mask & (*w)) && count<64; ++count; do
			(*w) <<= 1;
		end
		return(count);
	end

	declare_function(uint64, approx_high_uint64_word_of_uint64_mult_uint64, params(pointer(uint64), pointer(uint64), uint8));
	function(approx_high_uint64_word_of_uint64_mult_uint64)
			// Berechnet eine Nherung von floor(x * y / (2 hoch 64)). Wenn signed_mult==0 bergeben wird, ist
			// der zurckgegebene Wert ist niemals grer und maximal um 3 kleiner als der tatschliche Wert.
			// 0!=signed_mult & 1 : *x ist vorzeichenbehaftet
			// 0!=signed_mult & 2 : *y ist vorzeichenbehaftet
		param x(pointer(uint64)), y(pointer(uint64)), signed_mult(uint8);
		local r(uint64, ((*x)>>32)*((*y)>>32) + ((((*x)>>32)*((*y)&0xffffffff))>>32) + ((((*y)>>32)*((*x)&0xffffffff))>>32));
		if(0!=(signed_mult & 1) && (cast(*x, int64))<0)
			r -= (*y);
		end
		if(0!=(signed_mult & 2) && (cast(*y, int64))<0)
			r -= (*x);
		end
		return(r);
	end

	declare_function(uint64, approx_high_uint64_word_of_uint64_mult_uint64_pbv_y, params(pointer(uint64), uint64, uint8));
	function(approx_high_uint64_word_of_uint64_mult_uint64_pbv_y)
		param x(pointer(uint64)), y(uint64), signed_mult(uint8);
		return(approx_high_uint64_word_of_uint64_mult_uint64(x, &y, signed_mult));
	end

	declare_function(uint64, approx_inverse_of_fixpoint_uint64, params(pointer(uint64)));
	function(approx_inverse_of_fixpoint_uint64)
		// Es muss 0!=*y & 0x8000000000000000 sein !
		// Unter dieser Voraussetzung wird eine Nherung fr (2 hoch 126) / *y berechnet.
		// Theoretische berlegungen ergeben, dass nahezu alle 64 Bits des Ergebnisses signifkant sind.
		// Numerische Experimente lassen vermuten, dass der zurckgegebene Wert um maximal +/- 1 gegenber dem
		// auf die nchste ganze Zahl gerundeten Wert von (2 hoch 126) / *y abweicht.
		param y(pointer(uint64));
		local yv(uint64, *y), z8(uint64), k5(uint64);
		local z3(uint16);
		local z5(uint32, yv>>32), z6(uint32), k(uint32);
		local c(uint8);

		if(0!=((cast(yv, uint32))&0x80000000) && 0==++z5)
			z6=0;
		else
			z3=(z5>>16);

			if(0==(z3&0x7fff))
				if(0==(z5&0xffff))
					return((0xffffffffffffffffULL+1)-yv); // "Warning: integral constant overflow" OK
				end
				z6=-((z5&0xffff)<<2);
			else if(0xffff==z3)
				z6=-z5;
			else
				z3=cast(1 + (0xffffffff-(z3>>1)) / z3, uint16);
				z6=((cast(z3, uint32))<<17) - (z5 + ((z3*((0x20000+cast(z3, uint32))*cast(z5, uint64)))>>32))-1;
			end end
		end

		for k=(k5=(0x100000000ULL | (cast(z6, uint64)))*z5 + (z5>>1)), c=k5>>32 ; 0!=c ; _nop_; do
			if(k+z5<k) ++c; end // Nach einem 8-Bit-berlauf von c wird die Schleife abgebrochen
			k+=z5;
			++z6;
		end

		z8=((cast(1LU, uint64))<<63) | (cast(z6, uint64))<<31;

		// In der folgenden Anweisung wird ein Integer-berlauf bewusst ausgenutzt.
		// Es wird a=(2 hoch 64) -z -approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&z, yv<<1, 0) berechnet.
		k5 = (0xffffffffffffffffULL+1)-z8-approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&z8, yv<<1, 0); // Bei yv<<1 geht das hchstwertigste Bit verloren
		 // "Warning: integral constant overflow" OK
		z8=approx_high_uint64_word_of_uint64_mult_uint64(&z8, &k5, 0) + cond(0==(k5&0x8000000000000000ULL), z8, 0);
		return((z8+1)>>1);
	end

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	//declare_function(uint64, f_long_to_float64, params(int32));
	function(f_long_to_float64)
		param n(int32);
		local r(uint64), w(uint64, cast(cond(n<0 , -n , n), uint64)<<20);
		f_combi_from_fixpoint(&r, cond(n<0 , 1 , 0), 1023+32, &w);
		return(r);
	end

	//declare_function(uint64, f_abs, params(uint64));
	function(f_abs)
		param x(uint64);
		return(x & 0x7fffffffffffffffULL);
	end

	declare_void_function(f_addsub2, params(pointer(uint64), uint64, uint64, uint8, pointer(uint8)));
	void_function(f_addsub2)
		param x(pointer(uint64)), a(uint64), b(uint64), flagadd(uint8), flagexd(pointer(uint8));
		local sig(uint8), aex(int16), bex(int16), wa(uint64), wb(uint64);

		f_split64(&a,&sig,&aex,&wa, 10);
		f_split64(&b,&sig,&bex,&wb, 10);
		*flagexd=0;

		//printsu(copystr("aex = "), aex);
		//printsu(copystr("bex = "), bex);

#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==aex || 2047==bex)
			*x=float64_ONE_POSSIBLE_NAN_REPRESENTATION;
			return();
#else
		if(2047==aex) // a ist ein NaN oder +INF oder -INF
			if(0==wa && (2047!=bex || (0==wb && flagadd)))
				*x=a;
			else
				*x=float64_ONE_POSSIBLE_NAN_REPRESENTATION;
			end
			return();
		else if(2047==bex)
			*x=b;
			return();
		end end
#endif

		if(!aex || aex+64<=bex)
			*x=b; *flagexd=1; return(); // Alle denormalisierten Zahlen werden als Null interpretiert.
		end
		if(!bex || bex+64<=aex)
			*x=a; return(); // Alle denormalisierten Zahlen werden als Null interpretiert.
		end

		if(flagadd)
			if(aex>=bex)
				wa+=wb>>(aex-bex); // aex-bex<64 wurde schon durch die obige Abfrage if(!bex || bex+64<=aex) sichergestellt
			else
				wa=wb+(wa>>(bex-aex)); // bex-aex<64 wurde schon durch die obige Abfrage if(!aex || aex+64<=bex) sichergestellt
				aex=bex;
			end
		else
			if(aex>bex || (aex==bex && wa>=wb))
				wa-=wb>>(aex-bex); // aex-bex<64 wurde schon durch die obige Abfrage if(!bex || bex+64<=aex) sichergestellt
			else
				wa=wb-(wa>>(bex-aex)); // bex-aex<64 wurde schon durch die obige Abfrage if(!aex || aex+64<=bex) sichergestellt
				*flagexd=1;
				aex=bex;
			end
		end
		f_combi_from_fixpoint(x, 0, aex-10, &wa);
	end

	declare_void_function(f_setsign, params(pointer(uint64), int8));
	void_function(f_setsign)
		param x(pointer(uint64)), sign(int8);
		if(sign)
			*x |= 0x8000000000000000ULL;
		else
			*x &= 0x7fffffffffffffffULL;
		end
	end

	declare_function(uint8, f_getsign, params(uint64));
	function(f_getsign)
		param x(uint64);
		local px(pointer(uint64), &x);
		return(((*px)>>63)&1);
	end

	function(f_add)
		param a(uint64), b(uint64);
		local signa(uint8), signb(uint8), signerg(uint8), flagexd(uint8);
		local i64(uint64), x(pointer(uint64), &i64);

		signa=f_getsign(a);
		signb=f_getsign(b);
		if(signa^signb)  // diff. signs
			f_addsub2(x,a,b,0,&flagexd); // calc a-b
			signerg=((flagexd^signa))&1;  
		else
			f_addsub2(x,a,b,1,&flagexd); // calc a+b
			signerg= signa;
		end
		f_setsign(x,signerg);
		return(i64);
	end

	function(f_sub)
		param a(uint64), b(uint64);
		local signb(uint8), bloc(uint64, b), i64(uint64);

		signb= f_getsign(bloc);
		signb ^= 1;
		f_setsign(&bloc,signb);
		i64=f_add(a,bloc);
		return(i64);
	end

	function(f_mult)
		param fa(uint64), fb(uint64);
		local asig(uint8), bsig(uint8), aex(int16), bex(int16);
		local am(uint64), bm(uint64);

		f_split64(&fa,&asig,&aex,&am, 11);
		f_split64(&fb,&bsig,&bex,&bm, 11);

#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==aex || 2047==bex)
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		else if(!aex || !bex)  // gendert von Florian Knigstein: Alle denormalisierten Zahlen werden als Null interpretiert.
			return(float64_NUMBER_PLUS_ZERO);
		else
			aex=aex+bex-(0x3ff+10);
			am=approx_high_uint64_word_of_uint64_mult_uint64(&am, &bm, 0);
		end end
#else
		if(2047==aex) // a ist ein NaN oder +INF oder -INF
			if(0!=am || (2047==bex && 0!=bm) || 0==bex)
				return(float64_ONE_POSSIBLE_NAN_REPRESENTATION); // Auch +/-INF * Null ergibt NaN
			end
			// (+/- INF) * (+/- INF) oder (+/- INF) * endliche Zahl auer Null ergibt +/-INF
		else if(2047==bex)
			if(0!=bm || 0==aex)
				return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
			end
			aex=bex; am=bm;
		else if(!aex || !bex)  // gendert von Florian Knigstein: Alle denormalisierten Zahlen werden als Null interpretiert.
			return(float64_NUMBER_PLUS_ZERO);
		else
			aex=aex+bex-(0x3ff+10);
			am=approx_high_uint64_word_of_uint64_mult_uint64(&am, &bm, 0);
		end end end
#endif
		asig ^= bsig;
		f_combi_from_fixpoint(&fa, asig, aex, &am); // Hier darf von aex nichts subtrahiert werden, damit
		// auch aex==2047 (INF, NaN) nicht verflscht wird.
		return(fa);
	end

	function(f_div)
		param x(uint64), y(uint64);
		local xsig(uint8), ysig(uint8), xex(int16), yex(int16);
		local xm(uint64), ym(uint64), i64(uint64);

		f_split64(&x,&xsig,&xex,&xm, 11);
		f_split64(&y,&ysig,&yex,&ym, 11);
#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==xex || 2047==yex || 0==yex)
			return float64_ONE_POSSIBLE_NAN_REPRESENTATION;
		else
			i64=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&xm, approx_inverse_of_fixpoint_uint64(&ym), 0);
			xex += 1023-yex;
		end
#else
		if(0==yex)
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		else if(2047==xex) // x ist ein NaN oder +INF oder -INF
			if(0!=xm || 2047==yex)
				return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
			end
			// Ansonsten: (+/- INF) / (endliche Zahl auer Null)
			return(cond(xsig^ysig , float64_MINUS_INFINITY , float64_PLUS_INFINITY));
		else if(2047==yex) // x ist ein NaN oder +INF oder -INF
			if(0==ym) // endliche Zahl / INF = Null
				return(float64_NUMBER_PLUS_ZERO);
			end
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		else
			i64=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&xm, approx_inverse_of_fixpoint_uint64(&ym), 0);
			xex += 1023-yex;
		end end end
#endif
		f_combi_from_fixpoint(&x, xsig^ysig, xex-10, &i64);
		return(x);
	end

#define F_WITH_float64_to_long
#define F_WITH_isnan
#define F_WITH_finite
#define F_WITH_compare

#define F_WITH_cut_noninteger_fraction
#ifdef F_WITH_cut_noninteger_fraction
	//declare_function(uint64, f_cut_noninteger_fraction, params(uint64));
	function(f_cut_noninteger_fraction)
		param x(uint64);
		local f_ex(int16, (((x)>>52) & 2047));
		if(0==f_ex || f_ex>=1023+52) // Alle denormalisierten Zahlen werden als Null interpretiert.
			return(x); // schliet auch die Flle x=NaN, x=+/-INF ein.
		end
		if(f_ex<1023)
			return(float64_NUMBER_PLUS_ZERO);
		end
		return(x & (0xffffffffffffffffULL << ((1023+52)-f_ex)));
	end
#endif

#if defined(F_WITH_float64_to_long) || defined(F_WITH_exp)
	//declare_function(int32, f_float64_to_long, params(uint64));
	function(f_float64_to_long)
		param x(uint64);
			// Wenn f_abs(x) grer als 1LU<<31 ist, wird Null geliefert.
		local f_sign(uint8), f_ex(int16), w(uint64);
		f_split_to_fixpoint(&x, &f_sign, &f_ex, &w, 0);
		return(cond(f_ex>=1023+32 , 0 , cond(f_sign , -(cast(w, int32)) , (cast(w, int32))))); // Hiermit werden auch die Flle
			// x=+/-INF und x=NaN abgedeckt (Lieferung von 0). Im Fall x=0 ist w=0.
	end
#endif

#ifdef F_WITH_isnan
	//declare_function(uint8, f_isnan, params(uint64));
	function(f_isnan)
		param x(uint64);
		// Returns nonzero if x is an IEEE 754 "Not A Number" and otherwise zero.
		return(0x7ff0000000000000ULL == ((cast(x, uint64)) & 0x7ff0000000000000ULL) && 0!=((cast(x, uint64)) & 0xfffffffffffffULL));
	end
#endif

#ifdef F_WITH_finite
	//declare_function(uint8, f_finite, params(uint64));
	function(f_finite)
		param x(uint64);
	// Returns nonzero if x represents a real number and zero otherwise i.e. if x is +INF, -INF or NaN.
	return(0x7ff0000000000000ULL != ((cast(x, uint64)) & 0x7ff0000000000000ULL));
end
#endif

#ifdef F_WITH_compare
	//declare_function(int8, f_compare, params(uint64, uint64));
	function(f_compare)
		param x(uint64), y(uint64);
		// If both x and y represent real numbers
		// (or +/-INF if F_ONLY_NAN_NO_INFINITY is not defined) f_compare returns
		// zero if x is equal to y, positive nonzero if x > y and negative nonzero if x < y.
		// If x or y are NaN, f_compare returns zero.
		local asig(uint8), bsig(uint8), xex(int16), yex(int16), wx(uint64), wy(uint64);
		f_split64(&x,&asig,&xex,&wx, 0);
		f_split64(&y,&bsig,&yex,&wy, 0);
#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==xex || 2047==yex)
			return(0);
#else
		if((2047==xex && 0!=wx) || (2047==yex && 0!=wy)) // x=NaN oder y=NaN
			return(0);
		end
		if(2047==xex) // x = +INF oder -INF (oder NaN, aber schon oben abgefangen)
			if(2047==yex) // y = +INF oder -INF
				return(cond(asig==bsig , 0 , (cond(asig , -1 , 1))));
			else
				return(cond(asig , -1 , 1));
			end
		end
		if(2047==yex) // y = +INF oder -INF (oder NaN, aber schon oben abgefangen)
			return(cond(bsig , 1 , -1)); // x kann wegen den obigen Abfragen weder +/-INF noch NaN sein
		end
	#endif
		if(0==xex) // x = 0 (Alle denormalisierten Zahlen werden als Null interpretiert)
			return(cond((0==yex && 0==wy) , 0 , (cond(bsig , 1 , -1))));
		end
		if(0==yex || asig!=bsig || xex>yex)   // Alle denormalisierten Zahlen werden als Null interpretiert
			return(cond(asig , -1 , 1));
		end
		if(xex<yex)
			return(cond(asig , 1 , -1));
		end
		return(cond(wx==wy , 0 , cond((wx>wy && !asig) || (wx<wy && asig) , 1 , -1)));
	end
#endif

#define F_WITH_to_decimalExp
#if defined(F_WITH_to_decimalExp) || defined(F_WITH_to_string) || defined(F_WITH_strtod) || defined(F_WITH_atof)
	declare_function(int16, f_10HochN, params(int64, pointer(uint64)));
	function(f_10HochN)
		param n(int64), res(pointer(uint64));
		local pot(uint64, (cast(10, uint64))<<60);
		local exp2(int16, 0), pot_exp2(int16, 3), neg(uint8, 0);
		*res=(cast(1, uint64))<<63;
		if(n<0)
			neg=1; n=-n;
		end
		while(0 != n)
			if(0 != (n & 1))
				*res = approx_high_uint64_word_of_uint64_mult_uint64(res, &pot, 0);
				exp2+=pot_exp2+1-f_shift_left_until_bit63_set(res);
			end
			pot = approx_high_uint64_word_of_uint64_mult_uint64(&pot, &pot, 0);
			pot_exp2=(pot_exp2<<1)+1-f_shift_left_until_bit63_set(&pot);
			n >>= 1;
		end
		if(neg)
			*res = approx_inverse_of_fixpoint_uint64(res);
			exp2=-exp2-f_shift_left_until_bit63_set(res);
		end
		return(exp2);
	end
#endif

#if defined(F_WITH_to_decimalExp) || defined(F_WITH_to_string)
	function(f_to_decimalExp)
		param x(uint64), anz_dezimal_mantisse(uint8), MantisseUndExponentGetrennt(uint8), ExponentBasis10(pointer(int16));
		// f_to_decimalExp() converts the float64 to the decimal representation of the number x if x is
		// a real number or to the strings "+INF", "-INF", "NaN". If x is real, f_to_decimalExp() generates
		// a mantisse-exponent decimal representation of x using anz_dezimal_mantisse decimal digits for
		// the mantisse. If MantisseUndExponentGetrennt!=0 is passed f_to_decimalExp() will generate different
		// strings for the mantisse and the exponent. If you assign
		//      char *str=f_to_decimalExp(x, anz_mts, 1, 0)
		// then str points to the mantisse string and str+strlen(str) points to the exponent string.
		// If the pointer ExponentBasis10 passed to f_to_decimalExp() is nonzero, the function will
		// assign the 10-exponent to *ExponentBasis10 ; e.g. if the decimal representation of x
		// is 1.234E58 then the integer 58 is assigned to *ExponentBasis10.

		// *** ATTENTION *** : The functions f_to_decimalExp() and f_to_string() return pointers to static memory
		// containing the decimal representation of the float64 passed to these functions. The string contained
		// in this memory will become invalid if one of the functions f_to_decimalExp(), f_to_string(),
		// f_exp(), f_log(), f_sin(), f_cos(), f_tan(), f_arcsin(), f_arccos(), f_arctan() is called as
		// these functions will overwrite the memory.

		local f_sign(uint8), len(uint8), posm(uint8), i(uint8);
		local f_ex(int16), Exp10(int16), w(uint64), w2(uint64);

		if(anz_dezimal_mantisse>17)
			anz_dezimal_mantisse=17;
		end
		if(anz_dezimal_mantisse<1)
			anz_dezimal_mantisse=1;
		end
		f_split64(&x, &f_sign, &f_ex, &w, 11);
		if(0==f_ex) // Alle denormalisierten Zahlen werden als Null interpretiert.
			TemporaryMemory[0]='0'; TemporaryMemory[1]=0; return(&TemporaryMemory[0]);
		end
#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==f_ex)
			strcpy(TemporaryMemory, "NaN"); return(TemporaryMemory);
		end
#else
		if(2047==f_ex)
			if(0!=w)
				strcpy(&TemporaryMemory[0], "NaN"); return(&TemporaryMemory[0]);
			end
			TemporaryMemory[0]=cond(f_sign , '-' , '+');
			strcpy(&TemporaryMemory[1], "INF");
			return(&TemporaryMemory[0]);
		end
#endif
		f_ex-=1023; // Nach der Abfrage auf 0==f_ex und 2047==f_ex !
		len=0;
		if(f_sign)
			TemporaryMemory[len++]='-';
		end

		if(f_ex >= 0)
			Exp10=cast((((cast(f_ex, uint16))*10+31)>>5), uint16);
		else
			Exp10=(-(((cast((-f_ex), uint16))*9)>>5));
		end

		f_ex+=f_10HochN(-Exp10, &w2);
		w=approx_high_uint64_word_of_uint64_mult_uint64(&w, &w2, 0);
		f_ex+=1-f_shift_left_until_bit63_set(&w);

		while(f_ex<0)
			w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w, (cast(10, uint64))<<60, 0);
			f_ex+=4-f_shift_left_until_bit63_set(&w);
			Exp10--;
		end
		while(f_ex>=4 || (f_ex==3 && (w & 0xf000000000000000ULL)>=0xa000000000000000ULL))
			w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w, 0xccccccccccccccccULL, 0);
			f_ex-=3+f_shift_left_until_bit63_set(&w);
			Exp10++;
		end
		posm=len;
		++len; // Platz fr .
		while(0!=(anz_dezimal_mantisse--))
			TemporaryMemory[len]='0';
			if(f_ex>=0)
				TemporaryMemory[len] += (w>>(63-f_ex));
				w <<= 1+f_ex;
				f_ex=-1;
			end
			++len;
			w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w, (cast(10, uint64))<<60, 0);
			f_ex+=4-f_shift_left_until_bit63_set(&w);
		end
		if(f_ex>=0 && (w>>(63-f_ex)) >= 5)
			for i=len ; --i>posm ; _nop_; do
				if(TemporaryMemory[i]=='9')
					TemporaryMemory[i]='0';
				else
					++TemporaryMemory[i];
					break;
				end
			end
			if(i==posm)
				++Exp10;
				TemporaryMemory[++i]='1';
				while(++i<len)
					TemporaryMemory[i]='0';
				end
			end
		end
		TemporaryMemory[posm]=TemporaryMemory[posm+1];
		TemporaryMemory[posm+1]='.';
		if(MantisseUndExponentGetrennt)
			TemporaryMemory[len++]=0;
		end
		TemporaryMemory[len++]='E';
		if(Exp10>0)
			TemporaryMemory[len++]='+';
		end
		itoa10(Exp10, &TemporaryMemory[len]);
		if(0!=ExponentBasis10)
			*ExponentBasis10=Exp10;
		end
		return(&TemporaryMemory[0]);
	end
#endif

#define F_WITH_to_string
#ifdef F_WITH_to_string
	//declare_function(pointer(uint8), f_to_string, params(uint64, uint8, uint8));
	function(f_to_string)
		param x(uint64), max_nr_chars(uint8), max_leading_mantisse_zeros(uint8);
	// f_to_decimalExp() converts the float64 to the decimal representation of the number x if x is
	// a real number or to the strings "+INF", "-INF", "NaN". If x is real, f_to_decimalExp() generates
	// a decimal representation without or with mantisse-exponent representation depending on
	// what in more suitable or possible. If -1 < x < 1 the non-exponent representation is chosen if
	// there are less than max_leading_mantisse_zeros zeros after the decimal point.
	// In most cases f_to_string() will generate a string with maximal 'max_nr_chars' chars. If
	// necessary, f_to_string() reduces the number of decimal digits in the mantisse in order to
	// get a maximum string width of 'max_nr_chars' chars. If however max_nr_chars is so small that
	// even a mantisse of one digit and the corresponding exponent doesn't fit into 'max_nr_chars' chars,
	// the string returned will be longer than 'max_nr_chars' chars.

	// *** ATTENTION *** : The functions f_to_decimalExp() and f_to_string() return pointers to static memory
	// containing the decimal representation of the float64 passed to these functions. The string contained
	// in this memory will become invalid if one of the functions f_to_decimalExp(), f_to_string(),
	// f_exp(), f_log(), f_sin(), f_cos(), f_tan(), f_arcsin(), f_arccos(), f_arctan() is called as
	// these functions will overwrite the memory.

		local exp10(int16);
		local nrd(uint8, cond((0!=(x & 0x8000000000000000ULL)) , (max_nr_chars-1) , max_nr_chars)); // nrd: Zahl der zu berechnenden dezimalen Mantissestellen.
		local nrd_vor(uint8, nrd+1);
		local r(pointer(uint8), 0), j(uint8), k(uint8);
		for j=0; j<3 && nrd!=nrd_vor; j++; do
			if(nrd>nrd_vor)
				r=f_to_decimalExp(x, nrd_vor, 1, &exp10); // Nur exp10 berechnen
				break;
			end
			r=f_to_decimalExp(x, nrd, 1, &exp10); // Nur exp10 berechnen
			if(((x>>52)&2047)==2047 || 0==(x & 0x7ff0000000000000ULL))
				return(r);
			end
			if(exp10<-max_leading_mantisse_zeros-1)
				break;
			end
			nrd_vor=nrd;

			nrd=cond(f_getsign(x) , max_nr_chars-1 , max_nr_chars);
				// Diese Variable nrd muss sowohl hier als oben vor Beginn der Schleife initialisiert werden.
			if(exp10<0)
				nrd += exp10-1; // exp10-1 Zeichen durch 0.00 .... verbraucht.
			else if(exp10+1<nrd) // && exp10>=0 .
				--nrd; // Ein Zeichen durch Dezimalpunkt verbraucht.
					// Beispiel fr den Fall exp10==max_nr_chars-2 : Es sei max_nr_chars=4, exp10=2, Zahl=683.79426
					// ==> Es wird 684 dargestellt
					// (3 statt 4 Zeichen belegt, weil 683.8 zu viele Zeichen belegen wrde).
			end end
		end

		if(0!=(x & 0x7ff0000000000000ULL) && exp10>=-max_leading_mantisse_zeros && nrd>exp10)  // Darstellung ohne 10er-Exponent (E)
			if(f_getsign(x))
				++r;  // - Zeichen
			end
			if(exp10<0)
				for j=strlen(r) ; (j--)>0 && '0'==r[j] ; _nop_; do
					r[j]=0;
				end
				r[1]=r[0];
				for ++j ; j>0 ; j--; do
					r[j-exp10]=r[j];
				end
				r[0]='0';
				r[1]='.';
				for j=2; ++exp10<0; _nop_; do
					r[j++]='0';
				end
			else
				for j=1 ; j<=exp10 ; j++; do
					 if(0==(r[j]=r[j+1]))
						 while(j<=exp10)
							 r[j++]='0';
						 end
						 r[j]=0;
						 break;
					 end
				 end
				if(j+1<max_nr_chars && 0!=r[j])
					r[j]='.';
					for j=strlen(r) ; (j--)>0 && '0'==r[j] ; _nop_; do
						r[j]=0;
					end
					if('.'==r[j])
						r[j]=0;
					end
				else
					r[j]=0; // Notwendig fr den Fall j+1>=max_nr_chars
				end
			end
			if(f_getsign(x))
				--r;
			end
		else // Darstellung mit 10er-Exponent (E)
			j=max_nr_chars-4;
			if(f_getsign(x))
				--j;
			end
			exp10=(x>>52)&2047; // exp10 wird hier zur Aufnahme des binren Exponenten "missbraucht"
			if(exp10<1023) --j; end
			if(exp10>1023+34 || exp10<1023-34) --j; end
			if(j<1) j=1; end
			// Im Folgenden darf nicht while( (r=f_to_decimalExp(x, j, 0, 0)), strlen(r)>(uint8_t)max_nr_chars )
			// geschrieben werden (Weglassen eines Klammerpaars), denn while ist ein Makro, und das Komma zwischen
			// (r=f_to_decimalExp(x, j, 0, 0)) und strlen(r)>(uint8_t)max_nr_chars wrde dann nicht als Kommaoperator,
			// sondern als Trennparameter fr Makroparameter interpretiert.
			while( ((r=f_to_decimalExp(x, j, 0, 0)), strlen(r)>cast(max_nr_chars, uint8)) )
				if(--j<1)
					break;
				end
			end
			for j=2; 0!=r[j] && 'E'!=r[j] && 'e'!=r[j] ; j++; do
				;
			end
			k=j;
			while((--j>=4 || (!f_getsign(x) && j>=3)) && '0'==r[j]) end
			while(0!=(r[++j]=r[k++])) end
		end
		return(r);
	end
#endif

#define F_WITH_strtod
#if defined(F_WITH_strtod) || defined(F_WITH_atof)
	//declare_function(uint64, f_strtod, params(pointer(uint8), pointer(pointer(uint8))));
	function(f_strtod)
		param str(pointer(uint8)), endptr(pointer(pointer(uint8)));
		local s(str), f_sign(uint8, 0), point(uint8, 0), f_ex(int16), w(uint64), w2(uint64), Exp10(int16), x(uint64);
		while(*s && (*s==' ' || *s=='\t'))
			s++;
		end

		if((s[0]=='n' || s[0]=='N') && (s[1]=='a' || s[1]=='A') && (s[2]=='n' || s[2]=='N'))
			s+=3; goto(return_NaN);
		end

		if(*s=='-')
			f_sign=1;
			++s;
		else if(*s=='+')
			++s;
		end end

#ifndef F_ONLY_NAN_NO_INFINITY
		if((s[0]=='i' || s[0]=='I') && (s[1]=='n' || s[1]=='N') && (s[2]=='f' || s[2]=='F'))
			if(endptr) *endptr=s+3; end
			return(cond(f_sign , float64_MINUS_INFINITY , float64_PLUS_INFINITY));
		end
#endif

		Exp10=0;
		w=0; f_ex=3;
		while(1)
			if(*s=='.')
				++s;
				if(point)
					goto(return_NaN);
				end
				point=1;
			end
			if(!(*s>='0' && *s<='9'))
				break;
			end
			if(f_ex<=63-4)
				if(point)
					--Exp10;
				end
				w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w, (cast(10, uint64))<<60, 0);
				if(w)
					f_ex+=4;
				end
				w += ((cast(*s, uint64))-'0')<<(63-f_ex);
				if(w)
					f_ex-=f_shift_left_until_bit63_set(&w);
				end
			else if(!point)
				++Exp10;
			end end
			++s;
		end
		if(*s=='E' || *s=='e')
			Exp10+=atoi(++s);
			if(*s=='-' || *s=='+')
				++s;
			end
			while(*s>='0' && *s<='9')
				++s;
			end
		end

		f_ex+=1+f_10HochN(Exp10, &w2);
		w=approx_high_uint64_word_of_uint64_mult_uint64(&w, &w2, 0);

		f_combi_from_fixpoint(&x, f_sign, 1023-11+f_ex, &w);
		if(endptr) *endptr=s; end
		return(x);

		label(return_NaN);
		if(endptr) *endptr=s; end
		return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
	end
#endif

#define F_WITH_exp
#if defined(F_WITH_exp) || defined(F_WITH_log) || defined(F_WITH_sin) || defined(F_WITH_cos) || defined(F_WITH_tan) || defined(F_WITH_arcsin) || defined(F_WITH_arccos) || defined(F_WITH_arctan)
	//declare_function(uint64, f_eval_function_by_rational_approximation_fixpoint,
	//		params(uint64, uint8, uint8, pointer(uint64), uint8));
	function(f_eval_function_by_rational_approximation_fixpoint)
		param x(uint64), anz_zaehler(uint8), anz_nenner(uint8), koeffs(pointer(uint64)), signed_mult(uint8);
		// 0!=(signed_mult&1) : koeffs[] oder x sind vorzeichenbehaftet.
		// 0!=(signed_mult&2) : x wird als negativ interpretiert, darf aber NICHT als Zweierkomplement vorliegen
		// (es muss dann also der Betrag abs(x) von x bergeben werden).
		// Evaluation einer rationalen Funktion mit den Koeffizienten koeffs[] mittels Festkommaarithmetik.
		// Durch x wird die rationale Zahl xr = x / (2 hoch 64) dargestellt (==> Es kann nur 0<=xr<1 bergeben werden
		// oder aber -1<x<=0, falls 0!=(signed_mult&2)).
		// Der rationale Funktionswert muss zwischen 0 und kleiner als 4 liegen, falls 0==(signed_mult&1), und zwischen
		// grer als -2 und kleiner als 2, falls 0!=(signed_mult&1).
		// Er wird vor der Rckgabe mit 2 hoch 62 multipliziert, so dass 62 binre Nachkommastellen zur Verfgung stehen.
		// Ein negativer Rckgabewert wird im Zweierkomplement dargestellt.
		// Durch koeffs[n] wird jeweils eine rationale Zahl koeffs[n] / (2 hoch 62) dargestellt.
		// Wenn das hchstwertigste Bit von koeffs[n] gesetzt ist und 0!=(signed_mult&1) bergeben wird,
		// wird ein negativer Koeffizient im Zweierkomplement dargestellt.
		// Durch zz und nn wird jeweils eine rationale Zahl zz / (2 hoch 62) bzw. nn / (2 hoch 62) dargestellt.

		local zz(uint64), nn(uint64), sh(uint8);

		if(0==anz_zaehler)
			zz=0x800000000000000ULL;
		else
			for zz=0 ; (( cond((0==(signed_mult & 2) || 0!=(anz_zaehler&1)) , (zz+=*koeffs) , (zz-=*koeffs))), koeffs++, 0!=--anz_zaehler) ;
					_nop_; do
				zz=approx_high_uint64_word_of_uint64_mult_uint64(&zz, &x, signed_mult & 1);
			end
		end

		if(0==anz_nenner)
			return(zz);
		end

		for nn=0 ; (cond((0==(signed_mult & 2) || 0!=(anz_nenner&1)) , (nn+=*koeffs) , (nn-=*koeffs)), koeffs++, 0!=--anz_nenner) ;
				_nop_; do
			nn=approx_high_uint64_word_of_uint64_mult_uint64(&nn, &x, signed_mult & 1);
		end

		sh=0;
		while(0==(0xff00000000000000ULL & nn))
			// Fr approx_inverse_of_fixpoint_uint64() muss das hchstwertigste Bit gesetzt sein !
			nn<<=8;
			if(0==(0xff00000000000000ULL & zz))
				zz<<=8;
			else
				sh+=8;
			end
		end
		while(0==(0x8000000000000000ULL & nn))
			// Fr approx_inverse_of_fixpoint_uint64() muss das hchstwertigste Bit gesetzt sein !
			nn<<=1;
			if(0==(0x8000000000000000ULL & zz))
				zz<<=1;
			else
				++sh;
			end
		end
		nn=approx_inverse_of_fixpoint_uint64(&nn);
		return(approx_high_uint64_word_of_uint64_mult_uint64(&zz, &nn, 0) << sh);
	end
#endif

#if defined(F_WITH_fmod) || defined(F_WITH_exp) || defined(F_WITH_sin) || defined(F_WITH_cos) || defined(F_WITH_tan)
	declare_function(uint64, f_mod_intern, params(uint64, uint8, int16, pointer(uint64), pointer(uint64)));
	function(f_mod_intern)
		param x(uint64), ysig(uint8), yex(int16), ymts(pointer(uint64)), ganz(pointer(uint64));

		local xsig(uint8), count(uint8), xex(int16), zex(int16), xm(uint64), g(uint64), q(uint64);

		f_split64(&x,&xsig,&xex,&xm, 11);
		if(ganz) *ganz=float64_NUMBER_PLUS_ZERO;
			//printsu(copystr("GANZ = "), *ganz);
			//waitforkey();
		end
		if(0==xex)
			return(float64_NUMBER_PLUS_ZERO);
		end
		if(2047==xex || 2047==yex || 0==(yex|(*ymts)))
			if(ganz) *ganz=float64_ONE_POSSIBLE_NAN_REPRESENTATION; end
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end

		/*printsu(copystr("xex = "), xex);
		printsu(copystr("yex = "), yex);
		printsu(copystr("xm = "), xm);
		printsu(copystr("ymts = "), *ymts);*/
		for count=10 ; 0!=(xex|xm) && (xex>yex || (xex==yex && xm>=(*ymts))) && 0!=--count ; _nop_; do
			// In vielen Fllen wird diese Schleife kein oder ein Mal durchlaufen.
			q=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&xm, (approx_inverse_of_fixpoint_uint64(ymts)-5)<<1, 0)-5;
				// approx_inverse_of_fixpoint_uint64(&ym)-5 ist nicht grer als der tatschliche Wert (ohne Rundungsfehler).
				// Auerdem ist (2 hoch 62) <= approx_inverse_of_fixpoint_uint64(&ym)-5 < (2 hoch 63).
			if(xex==yex && 0==(0x8000000000000000ULL & q)) // Rundungsfehler
				q=0x8000000000000000ULL; // entspricht der Zahl 1
			end
			zex=xex-yex;
			if(zex<=63)
				q &= (0xffffffffffffffffULL << (63-zex));
			end
			xm -= approx_high_uint64_word_of_uint64_mult_uint64(&q, ymts, 0)<<1;

			if(ganz)
				//printsu(copystr("q = "), q);
				f_combi_from_fixpoint(&g, xsig^ysig, (1023-11)+zex, &q);
				//printsu(copystr("ganz_vor = "), *ganz);
				//printsu(copystr("g = "), g);
				//printsu(copystr("ganz add = "), f_add(*ganz, g));
				*ganz=f_add(*ganz, g);
				//printsu(copystr("ganz_nach = "), *ganz);
			end
			xex-=f_shift_left_until_bit63_set(&xm);
		end
		f_combi_from_fixpoint(&x, xsig, xex-11, &xm);
		*ymts=xm;
		if(0==count)
			return(float64_NUMBER_PLUS_ZERO); // Keine exakte Berechnung erfolgt. Sicherstellen, dass
				// der Rckgabewert innerhalb des Bereichs von 0 bis ausschlielich y liegt.
		end
		return(x);
	end
#endif


#define F_WITH_fmod
#ifdef F_WITH_fmod
	//declare_function(uint64, f_mod, params(uint64, uint64, pointer(uint64)));
	function(f_mod)
			param x(uint64), y(uint64), ganz(pointer(uint64));
		// The fmod function calculates the floating-point remainder f of x / y such that x = i * y + f,
		// where i is an integer, f has the same sign as x, and the absolute value of f is less than the absolute value of y.
		local ysig(uint8), yex(int16), ym(uint64);
		f_split64(&y,&ysig,&yex,&ym, 11);
		return(f_mod_intern(x, ysig, yex, &ym, ganz));
	end
#endif

#define F_WITH_sqrt
#if defined(F_WITH_sqrt) || defined(F_WITH_sin) || defined(F_WITH_cos) || defined(F_WITH_tan) || defined(F_WITH_arcsin) || defined(F_WITH_arccos) || defined(F_WITH_arctan)
	declare_function(uint64, rounded_sqrt_of_integer128, params(uint64, uint64));
	function(rounded_sqrt_of_integer128)
		param x_high(uint64), x_low(uint64);
		// Liefert die auf die nchste ganze Zahl gerundete Quadratwurzel aus (2 hoch 64)*x_high + x_low .
		// Falls die Wurzel nach Rundung gleich (2 hoch 64) sein msste, wird (2 hoch 64)-1 zurckgegeben.
		local rL(uint64, 0), rH(uint64, 0), sH(uint64), sL(uint64), i(uint8);
		for i=2*64; 0!=i; _nop_; do
			i-=2;
			sH=rH; sL=rL;
			if(i>=64)
				sH += ((UINT64)1LU) << (i-64);
			else
				if(sL >= cast(-(((UINT64)1LU) << i), uint64)) // sL >= (2 hoch 64) - (2 hoch i)
					sH++;
				end
				sL += ((UINT64)1LU) << i;
			end

			rL >>= 1;
			if(rH & 1)
				rL |= ((UINT64)1LU) << 63;
			end
			rH >>= 1;

			if(sH<x_high || (sH==x_high && sL<=x_low))
				x_high -= sH;
				if(sL > x_low)
					--x_high;
				end
				x_low -= sL;
				if(i>=64)
					rH |= ((UINT64)1LU) << (i-64);
				else
					rL |= ((UINT64)1LU) << i;
				end
			end
		end
		return(cond((0!=x_high || x_low>=rL) && 0xffffffffffffffffULL!=rL , rL+1 , rL));
	end
#endif

#ifdef F_WITH_sqrt
	//declare_function(uint64, f_sqrt, params(uint64));
	function(f_sqrt)
		param x(uint64);
		local xsig(uint8), xex(int16), w(uint64), wL(uint64);

		f_split64(&x,&xsig,&xex,&w, 11);
		if(0==xex) // Null
			return(float64_NUMBER_PLUS_ZERO);
		end
#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==xex || 0!=xsig)
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end
#else
		if(2047==xex) // NaN, +INF oder -INF
			if(0==w && 0==xsig) // +INF
				return(x);
			end
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end
		if(xsig)
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end
#endif

		wL=0;
		if(xex & 1)
			if(w & 1)
				wL = ((UINT64)1LU) << 63;
			end
			w >>= 1;
		end

		w=rounded_sqrt_of_integer128(w, wL);
		f_combi_from_fixpoint(&x, 0, (xex+1)/2 + 500, &w);
		return(x);
	end
#endif

#ifdef F_WITH_exp
	function(f_exp)
		param x(uint64);
		local f_sign(uint8), f_sign2(uint8), f_ex(int16), g(uint64), rest(uint64), gi(int16), w(uint64), w2(uint64);

		data_initialization data[]=
		{	// Interpolation von exp(x) im Bereich von x=0 ... log(2.0)
			// Zhler: 12 Koeffizienten, Nenner: 0 Koeffizienten
			0x00000025f42ff241ULL, 0x00000112e79387f1ULL, 0x00000ba525a70ff3ULL, 0x000067f7ce0d91cbULL,
			0x0003403a635c5891ULL, 0x0016c16a4128d5eeULL, 0x00888888e10886edULL, 0x02aaaaaaa0251d3eULL,
			0x0aaaaaaaab66a1c0ULL, 0x1ffffffffff9434dULL, 0x400000000000180bULL, 0x3ffffffffffffff2ULL,
			data_end_marker()
		};
		const interp(array(uint64, 12), data);
		
		f_split64(&x, &f_sign, &f_ex, &w, 0);

		if(0==f_ex)
			return(float64_NUMBER_ONE);
		end
#ifdef F_ONLY_NAN_NO_INFINITY
		// Ohne die folgende Abfrage wrde die unten stehende Funktion f_float64_to_long() einen falschen Wert liefern.
		// Damit werden auch die Flle erfasst, dass x=NaN oder x=+/-INF ist.
		if(f_ex>=1023+10)
			return(cond((f_sign && f_ex!=2047) , float64_NUMBER_PLUS_ZERO , float64_ONE_POSSIBLE_NAN_REPRESENTATION));
		end
#else
		if(2047==f_ex) // NaN, +INF oder -INF
			if(0!=w || (0==w && 0==f_sign)) // NaN oder +INF
				return(x);
			end
			return(float64_NUMBER_PLUS_ZERO);
		end
		if(f_ex>=1023+10) // Ohne diese Abfrage wrde die unten stehende Funktion f_float64_to_long() einen falschen Wert liefern.
			return(cond(f_sign , float64_NUMBER_PLUS_ZERO , float64_PLUS_INFINITY));
		end
#endif

		w=0xb17217f7d1cf79acULL; // Mantisse von log(2)
		rest=f_mod_intern(f_abs(x), 0, 1022, &w, &g); // Modulo log(2)
		// Von f_mod_intern() wurde an w nun die 64-Bit-Mantisse von x zugewiesen.

		f_split_to_fixpoint(&rest, &f_sign2, &f_ex, &w2, 64); // w2, nicht w bergeben !
		if(f_ex<=1022-64)
			w=0;
		else
			w>>=1022-f_ex;
		end
		gi=f_float64_to_long(g);

		if(f_sign2)
			w=0;
		else if(f_ex>=1023) // Rundungsfehler bei f_mod
			w=0xffffffffffffffffULL;
		end end

		w=f_eval_function_by_rational_approximation_fixpoint(w, 12, 0, &interp[0], 0);

		if(f_sign)
			gi-=f_shift_left_until_bit63_set(&w);
			w=approx_inverse_of_fixpoint_uint64(&w);
		end
		f_combi_from_fixpoint(&g, 0, cond(f_sign , (1021-10)-gi , (1023-10)+gi), &w);

		return(g);
	end
#endif


#define F_WITH_log
#ifdef F_WITH_log
	function(f_log)
		param x(uint64);
		local f_sign(uint8), sgnw2(int8, 0), f_ex(int16), f_ex2(int16), w(uint64), w2(uint64);

		data_initialization data[]=
		{	// Interpolation von log(1+x)/x im Bereich von x= -0.125 ... 1
			// Zhler: 7 Koeffizienten, Nenner: 8 Koeffizienten
			0x00020306b29459c0ULL, 0x0059c8a974126d6aULL, 0x03f628260c2db8e4ULL, 0x11dfa5d2d89584a5ULL,
			0x255cf1f4fada2dccULL, 0x243348927d4a6bdfULL, 0x0d22e971210e91e5ULL, 0x00005e30ad9c1880ULL,
			0x001a68f2d08c85a8ULL, 0x01b09953e747fe7eULL, 0x0af24b95490068eeULL, 0x2215f7d0824727dbULL,
			0x365e581f76be278eULL, 0x2ac4bd4b0dd1b53cULL, 0x0d22e971210e91e6ULL,
			data_end_marker()
		};
		const interp(array(uint64, 15), data);

		f_split64(&x, &f_sign, &f_ex, &w, 11);

#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==f_ex || 0==f_ex || 0!=f_sign)
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end
#else
		if(0==f_ex) // Null
			return(float64_MINUS_INFINITY);
		end
		if(2047==f_ex || 0!=f_sign) // NaN, +INF oder -INF
			if(0==w && 0==f_sign) // +INF
				return(x);
			end
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION); // -INF
		end
#endif

		if(1022==f_ex && w>=0xe000000000000000ULL)
			// Hiermit wird erreicht, dass auch im Fall, dass x kleiner als 1, aber dicht bei 1 liegt, alle
			// Mantissebits des berechneten Logarithmus signifikant sind (wenn man davon absieht, dass in seltenen
			// Fllen in die "falsche" Richtung gerundet wird).
			w=(0xffffffffffffffffULL+1)-w; // 0xffffffffffffffff+1 ergibt gewollt 0 (warning: integral constant overflow).
			 // "Warning: integral constant overflow" OK
			sgnw2=-1;
		else
			w2=f_ex-1023; // vor dem berschreiben von f_ex !
			if(f_ex<1023)
				w2=-w2;
				sgnw2=-2;
			else if(f_ex>1023)
				sgnw2=1;
			end end
			w<<=1; // Das hchstwertigste Bit hinausschieben: Entspricht der Subtraktion von 1 von der Mantisse.
		end

		f_ex = (1022-9) - f_shift_left_until_bit63_set(&w);

		if(0!=w)
			w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w,
				f_eval_function_by_rational_approximation_fixpoint(w >> ((1022-9)-f_ex), 7, 8, &interp[0], cond(-1==sgnw2 , 3 , 0)), 0);
		end

		if(0==sgnw2 || -1==sgnw2)
			f_ex2=f_ex;
		else
			f_ex2=(1023+63-11)-f_shift_left_until_bit63_set(&w2);
			w2=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w2, 0xb17217f7d1cf79acULL, 0);
			if(f_ex2>=f_ex)
				cond((f_ex2-f_ex>=64) , w=0 , (w>>=(f_ex2-f_ex)));
			else
				w<<=(f_ex-f_ex2);
			end
			if(sgnw2>=0)
				w+=w2;
			else
				w=w2-w;
			end
		end

		f_combi_from_fixpoint(&x, cond(sgnw2<0 , 1 , 0), f_ex2, &w);
		return(x);
	end
#endif

#define F_WITH_sin
#if defined(F_WITH_sin) || defined(F_WITH_cos) || defined(F_WITH_tan)
	//declare_function(uint64, f_berechne_sinus_cosinus_tangens, params(uint64, uint8));
	function(f_berechne_sinus_cosinus_tangens)
		param x(uint64), fkt_nr(uint8);
		// fkt_nr=0 ==> Sinus, fkt_nr=1 ==> Cosinus, fkt_nr=2 ==> Tangens
		local f_sign(uint8), f_ex(int16), w(uint64), w2(uint64), neg(uint8), tneg(uint8, 0);
		local mir45(uint8, cond((1==fkt_nr) , 1 , 0));

		data_initialization data[]=
		{	// Interpolation von sin(sqrt(x))/sqrt(x) im Bereich von x=0 ... (Pi/4)*(Pi/4)
			// Zhler: 7 Koeffizienten (vorzeichenbehaftet), Nenner: 0 Koeffizienten
			0x000000002bb08115ULL, 0xffffffe51a1d4a39ULL, 0x00000b8ef1a8f972ULL, 0xfffcbfcbfcc8da4aULL,
			0x0088888888879853ULL, 0xf555555555555eb1ULL, 0x3ffffffffffffff0ULL,
			data_end_marker()
		};
		const interp(array(uint64, 7), data);

		f_split64(&x, &f_sign, &f_ex, &w, 0);

		if(0==f_ex) // Null
			return(cond(1==fkt_nr , float64_NUMBER_ONE , float64_NUMBER_PLUS_ZERO));
		end
		if(2047==f_ex) // NaN, +INF oder -INF
			return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
		end

		tneg=neg=cond((0!=f_sign && 1!=fkt_nr) , 1 , 0);
		w2=0xc90fdaa22168c235ULL; // Mantisse von 2*PI
		x=f_mod_intern(f_abs(x), 0, 1023+2, &w2, 0); // Modulo 2*PI
		// Von f_mod_intern() wurde an w2 nun die 64-Bit-Mantisse von x zugewiesen.
		f_split_to_fixpoint(&x, &f_sign, &f_ex, &w, 61); // &w, nicht &w2 bergeben! w2 hat 64 Bits, w nur 53 Bits Mantisse

		w=w2; // bleibt im Fall f_ex<1022 unten erhalten
		if(1025-64>=f_ex)
			w2=0;
		else
			w2>>=1025-f_ex;
		end

		if(w2 >= 0x6487ed5110b4611aULL) // x>=PI	
			neg ^= 1;
			w2-=0x6487ed5110b4611aULL; // x -= PI
		end

		if(w2 >= 0x3243f6a8885a308dULL) // x>=PI/2	
			tneg ^= 1;
			if(1==fkt_nr)
				neg ^= 1;
			end
			w2=0x6487ed5110b4611aULL-w2; // x = PI-x
		end

		if(w2 >= 0x1921fb54442d1846ULL) // x>=PI/4	
			mir45 ^= 1;
			w2=0x3243f6a8885a308dULL-w2; // x = PI/2-x
		end

		w2<<=3;
		if(f_ex>=1022)
			w=w2;
			f_ex=0;
		else
			f_ex-=1022;
		end

		f_ex-=f_shift_left_until_bit63_set(&w); // Im Fall w=0 ist f_ex im Folgenden irrelevant

		w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w,
				f_eval_function_by_rational_approximation_fixpoint(
				approx_high_uint64_word_of_uint64_mult_uint64(&w2, &w2, 0), 7, 0, &interp[0], 1), 0);

		if(0!=(0x8000000000000000ULL & w)) // negativ durch Rundungsfehler
			w=0;
		end
		// Durch die vorangehende Abfrage ist sichergestellt, dass durch Rundungsfehler kein negatives w geliefert wird.
		// Das maximale w an dieser Stelle entspricht der reellen Zahl 1/sqrt(2) als Funktionswert von sin(Pi/4).

		if(mir45 || 2==fkt_nr)
			w2=w;
			if(f_ex<=-64)
				w=0;
			else
				w>>=(-f_ex);
			end
			w=rounded_sqrt_of_integer128(0xffffffffffffffffULL - (approx_high_uint64_word_of_uint64_mult_uint64(&w, &w, 0)<<4),
										 0xffffffffffffffffULL);
			// Hier darf nicht (0xffffffffffffffff+1) statt 0xffffffffffffffff stehen (Integer-berlauf), weil
			// die zu subtrahierende Zahl (approx_high_uint64_word_of_uint64_mult_uint64(&w, &w, 0)<<4)) auch Null sein kann.
			// Es ist sichergestellt, dass nicht durch Rundungsfehler ein w > 2 hoch 32 geliefert wird, welches
			// einem Funktionswert von sin bzw. cos von betraglich grer als 1 entsprechen wrde.
			if(2==fkt_nr)
				local ps(pointer(uint64)), pc(pointer(uint64));
				f_ex+=2;
				if(mir45)
					ps=&w; pc=&w2; f_ex=-f_ex;
				else
					ps=&w2; pc=&w;
				end
				if(0==*pc) // z.B. bei tan(PI/2)
					return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
				end
				f_ex-=f_shift_left_until_bit63_set(ps);
				f_ex+=f_shift_left_until_bit63_set(pc);
				w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(ps, approx_inverse_of_fixpoint_uint64(pc), 0);
				neg=tneg;
			else
				f_ex=-2;
			end
		end

		f_combi_from_fixpoint(&x, neg, 1013+f_ex, &w);
		return(x);
	end
#endif

#define F_WITH_arcsin
#if defined(F_WITH_arcsin) || defined(F_WITH_arccos) || defined(F_WITH_arctan)
	//declare_function(uint64, f_berechne_arcsin_arccos_arctan, params(uint64, uint8));
	function(f_berechne_arcsin_arccos_arctan)
		param x(uint64), fkt_nr(uint8);
		local f_sign(uint8), mir45(uint8, 0), nr_shift(uint8), f_ex(int16), w(uint64), w2(uint64);

		data_initialization data[]=
		{	// Interpolation von arctan(sqrt(x))/sqrt(x) im Bereich von x=0 ... 1
			// Zhler 7 Koeffizienten (alle positiv), Nenner 7 Koeffizienten (alle positiv)
			0x000051f590de71e7ULL, 0x00252120c9d0cdc1ULL, 0x0277897981ffc250ULL, 0x0e45a25bb3bfe210ULL,
			0x2360ae1e927608adULL, 0x26d3fc5ec2d01988ULL, 0x0f7f3182c822dbc8ULL, 0x000347839b4bdad9ULL,
			0x0082e7b5e08e6b4dULL, 0x056b67d2062eb49dULL, 0x1755b06e64aa19e5ULL, 0x2ef15e8d8e6c5b98ULL,
			0x2bfe6235058656aeULL, 0x0f7f3182c822dbceULL,
			data_end_marker()
		};
		const interp(array(uint64, 14), data);

		f_split64(&x, &f_sign, &f_ex, &w, 11);

		if(0==f_ex) // Null
			return(cond(1==fkt_nr , ((UINT64)0x3ff921fb54442d18ULL) , 0));
		end
#ifdef F_ONLY_NAN_NO_INFINITY
		if(2047==f_ex)
			return(x); // NaN
		end
#else
		if(2047==f_ex) // NaN, +INF oder -INF
			if(0!=w) // NaN
				return(x);
			end
			if(f_sign) // -INF
				return(cond(2==fkt_nr , ((UINT64)0xbff921fb54442d18ULL) , float64_ONE_POSSIBLE_NAN_REPRESENTATION));
			else
				return(cond(2==fkt_nr , ((UINT64)0x3ff921fb54442d18ULL) , float64_ONE_POSSIBLE_NAN_REPRESENTATION));
			end
		end
#endif

		if(2!=fkt_nr)
			if(f_ex>=1023 || (f_ex==1022 && w>=0xb504f333f9de6484ULL-0x0000000100000000ULL)) // fabs(x) >= sqrt(2)/2 - (2 hoch (-32))
					// Es ist wichtig, dass auf w>=0xb504f333f9de6484-0x0000000100000000 statt auf w>=0xb504f333f9de6484
					// getestet wird, damit w2 im else-Teil vor Ausfhrung von w2=approx_inverse_of_fixpoint_uint64(&w2)
					// mindestens gleich 0x8000000000000000 ist (wie von approx_inverse_of_fixpoint_uint64() gefordert).
				if(f_ex>1023 || (f_ex==1023 && w>0x8000000000000000ULL))
					return(float64_ONE_POSSIBLE_NAN_REPRESENTATION);
				end
				if(f_ex==1023) // fabs(x)=1 . Dieser Fall wrde im else-Code nicht korrekt behandelt.
					w=0;
					f_ex=1021;
				else
					// Die folgende Rechnung entspricht y=sqrt(2*(1-x) - (1-x)*(1-x)) / x .
					// Theoretisch (wenn ohne Rundungsfehler) liefert y=sqrt(1-x*x) / x das gleiche Ergebnis; aber wenn
					// x nahe bei 1 ist, muss zur Verringerung von Rundungsfehlern die erste Rechnung benutzt werden.
					w2=(0xffffffffffffffffULL+1) - w; // 0xffffffffffffffff+1 ergibt gewollt 0 (warning: integral constant overflow).
					 // "Warning: integral constant overflow" OK
					for nr_shift=0 ; 0==(0x6000000000000000ULL & w2) ; ++nr_shift; do
						w2<<=2;
					end
					w=approx_inverse_of_fixpoint_uint64(&w);
					w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w,
						rounded_sqrt_of_integer128((w2<<1) - (approx_high_uint64_word_of_uint64_mult_uint64(&w2,&w2,0)>>(2*nr_shift)), 0), 0);
					f_ex=1024-nr_shift;
				end
				mir45=cond((0==fkt_nr) , 1 , 0);
			else // fabs(x) < sqrt(2)/2 - (2 hoch (-32))
				f_split_to_fixpoint(&x, &f_sign, &f_ex, &w2, 64);
				w2=0xffffffffffffffffULL - approx_high_uint64_word_of_uint64_mult_uint64(&w2,&w2,0);
					// Obwohl w2=0 oben abgefangen wird, kann approx_high_uint64_word_of_uint64_mult_uint64(&w2,&w2,0)==0 sein.
					// Daher darf dieses Produkt nicht von (0xffffffffffffffff+1) abgezogen werden (Integer-berlauf).
				// Wegen fabs(x) < sqrt(2)/2 - (2 hoch (-32)) ist an dieser Stelle w2>=0x8000000000000000
				w2=approx_inverse_of_fixpoint_uint64(&w2);
				w2=rounded_sqrt_of_integer128(w2, 0);
				w=approx_high_uint64_word_of_uint64_mult_uint64(&w, &w2, 0); // Das hchstwertigste Bit kann gesetzt sein !
				++f_ex;
				mir45=cond((1==fkt_nr) , 1 , 0);
			end
		end

		if(0!=w) // Wenn fkt_nr!=2 und x=+1.0 oder x=-1.0 ist, wird oben w=0 gesetzt.
			f_ex-=f_shift_left_until_bit63_set(&w);
		end
		--f_ex;
		
		if(f_ex>=1022)
			w=approx_inverse_of_fixpoint_uint64(&w);
			f_ex=2044-f_ex-f_shift_left_until_bit63_set(&w);
			mir45=1;
		end

		// Im Folgenden muss der Fall 1022==f_ex gesondert behandelt werden, weil der dem Wert 1 entsprechende Wert nicht
		// an f_eval_function_by_rational_approximation_fixpoint() bergeben werden kann. Auerdem wrden sich sonst negative
		// Anzahl an Shifts ergeben ( w>>(1021-f_ex) ). Der Fall 1022==f_ex tritt bei arctan(1) und arctan(-1) und bei
		// bestimmten Werten von arccos und arcsin auf.
		if(1022==f_ex)
			w=0x1921fb54442d1847ULL; // ergibt x=Pi/4 oder x=-Pi/4
		else
			w=approx_high_uint64_word_of_uint64_mult_uint64_pbv_y(&w,
					f_eval_function_by_rational_approximation_fixpoint(cond(f_ex<=1021-32 , 0 ,
					(approx_high_uint64_word_of_uint64_mult_uint64(&w, &w, 0)>>(2*(1021-f_ex)))), 7, 7, &interp[0], 0), 0);
			if(mir45)
				w=0x6487ed5110b4611aULL - (cond(f_ex<=1021-64 , 0 , (w>>(1021-f_ex)))); // entspricht x = PI/2 - x
				f_ex=1021;
			end
			if(1==fkt_nr && f_sign)
				w=0xc90fdaa22168c235ULL - (cond(f_ex<=1021-64 , 0 , (w>>(1021-f_ex)))); // entspricht x = PI - x
				f_ex=1021;
				f_sign=0;
			end
		end

		f_combi_from_fixpoint(&x, f_sign, f_ex-8, &w);

		return(x);
	end
#endif


END_GLOBAL_SCOPE

#include "language_undefines.h"

void main2();

int main()
{
    // main2(); return 0;

	printf("Starte Kompilierung ...\n");
	BytecodeTargetedCompiler::Compiler compiler(TestFloat64Bibliothek);
	nibble_codesize=compiler.nibble_codesize;
	CodeOfProgram=compiler.bytecode;
	printf("\nKompilierung fertig.\n\n");

	printf("code size = %u bytes = 0x%x bytes = %u nibbles = 0x%x nibbles\r\n\r\n", (compiler.nibble_codesize+1)/2,
			(compiler.nibble_codesize+1)/2, compiler.nibble_codesize, compiler.nibble_codesize);
	printf("required memory size=%u bytes\r\n", compiler.required_memory_size);
	printf("code_begin=%u   =0x%x\r\n\r\n", compiler.function_begin_addresses["Main"]);

	FILE *file=fopen("d:\\program.txt", "wb");
	FILE *file2=fopen("d:\\program.bin", "wb");

	if(file && file2)
	{
		fprintf(file, "code_begin=%u   =0x%x\r\n\r\n", compiler.function_begin_addresses["Main"], compiler.function_begin_addresses["Main"]);
		fprintf(file, "code size = %u bytes = %x bytes = %u nibbles = %x nibbles\r\n\r\n", (compiler.nibble_codesize+1)/2,
			(compiler.nibble_codesize+1)/2, compiler.nibble_codesize, compiler.nibble_codesize);
    	fprintf(file, "required memory size=%u bytes\r\n", compiler.required_memory_size);
    	fprintf(file, "\r\nBytecode:\r\n", compiler.required_memory_size);
		for(uint32 i=0; i<(compiler.nibble_codesize+1)/2; i++)
		{
			fprintf(file, "%3u, ", CodeOfProgram[i]);
			if(19==(i%20)) fprintf(file, "\r\n");

			fwrite(&CodeOfProgram[i], 1, 1, file2);
		}
    	fprintf(file, "\r\nEnde des Bytecodes.\r\n\r\n", compiler.required_memory_size);
		fflush(file);
	}

	const uint16 stack_size=512, memory_size=1024;
	static uint8 stack[stack_size], memory[memory_size];

	BytecodeInterpreter::BytecodeInterpreter ip(stack, stack_size, memory, memory_size, get_code_nibble, get_code_byte, &extern_function_invoker);
	PROGRAM_COUNTER_TYPE code_begin=compiler.function_begin_addresses["Main"];

	try
	{
		ip.execute_bytecode(code_begin);
	}
	catch(BytecodeInterpreter::RuntimeError err)
	{
		err;
		printf("\n\n***** Laufzeit FEHLER *****\n\n");
		printf("===> %s\n", err.message);
		printf("===> PC = 0x%x\n\n", err.pc);
		while(_getch()==0);
		return 0;
	}

#ifdef CALCULATE_MAXIMUM_STACK_USAGE
	printf("Maximaler benutzter Stackspeicher: %u Bytes\n", ip.get_maximum_stack_usage());
	fprintf(file, "\r\nMaximaler benutzter Stackspeicher: %u Bytes\r\n", ip.get_maximum_stack_usage());
#endif
	if(file) fclose(file);
	if(file2) fclose(file2);
	while(_getch()==0);
	return 0;
}

static uint8 prgmem[]=
{	// Der Bytecode (vorcompiliert): Implementiert und testet die 64-Bit-Gleitkommabibliothek
118,   0,  17,   0, 209,   3, 152,   7, 254,   0,  20,   1,  42,   1, 160,   0, 177,   0,  12,  61, 
 46,   0,   0,   0,   0,   0,   0,   0,   0,   0, 240,   8,  32,   9,  20,  10, 204,  10, 204,  10, 
204,  10, 204,  10, 204,  10, 204,  10,  21,  39,  43,  56,  11,  14,  11, 114,  10,  26,   9,   0, 
  0,   1,   0,   1,   0,   1,   0,   1,   0,   1,   0,   1,   0,   1,   0,  17,   0,  17,   0,  17, 
  0,  17,   0,  17,   0,  17,   0,  17,   0, 177,   0, 177,   0, 177,   0, 177,   0, 177,   0, 177, 
  0, 209,   3, 209,   3, 209,   3, 209,   3, 209,   3,  24,   6,  24,   6,  24,   6,  24,   6,  24, 
  6,  24,   6, 152,   7, 152,   7,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48, 
 48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48, 
  2,   1,   0,   0,   0,   2,   0,   0,   0,   4,   0,   0,   0,   8,   0,   0,   0,  49,  50,  51, 
 46,  57,  56,  55,  32,  43,  32,  55,  51,  49,  50,  46,  53,  52,  51,  32,  61,   0,  49,  50, 
 51,  46,  55,  55,  55,  32,  47,  32,  52,  53,  54,  46,  49,  49,  49,  32,  61,   0, 115, 105, 
110,  40,  49,  50,  51,  46,  52,  52,  52,  41,  61,   0,  13,  10,  83, 116,  97, 114, 116, 101, 
 32,  49,  48,  48,  32,  65, 100, 100, 105, 116, 105, 111, 110, 101, 110,  32, 109, 105, 116,  32, 
 90, 117, 102,  97, 108, 108, 115, 122,  97, 104, 108, 101, 110,  32,  46,  46,  46,  13,  10,   0, 
 49,  48,  48,  32,  65, 100, 100, 105, 116, 105, 111, 110, 101, 110,  32, 102, 101, 114, 116, 105, 
103,  46,  13,  10,  13,  10,   0,  83, 116,  97, 114, 116, 101,  32,  49,  48,  48,  32,  68, 105, 
118, 105, 115, 105, 111, 110, 101, 110,  32, 109, 105, 116,  32,  90, 117, 102,  97, 108, 108, 115, 
122,  97, 104, 108, 101, 110,  32,  46,  46,  46,  13,  10,   0,  49,  48,  48,  32,  68, 105, 118, 
105, 115, 105, 111, 110, 101, 110,  32, 102, 101, 114, 116, 105, 103,  46,  13,  10,  13,  10,   0, 
 83, 116,  97, 114, 116, 101,  32,  53,  48,  32,  83, 105, 110, 117, 115,  45,  87, 101, 114, 116, 
101,  32, 118, 111, 110,  32,  90, 117, 102,  97, 108, 108, 115, 122,  97, 104, 108, 101, 110,  32, 
 46,  46,  46,  13,  10,   0,  53,  48,  32,  83, 105, 110, 117, 115,  45,  87, 101, 114, 116, 101, 
 32, 102, 101, 114, 116, 105, 103,  46,  13,  10,  13,  10,   0,  84, 101, 115, 116, 101,  32,  54, 
 52,  32,  66, 105, 116,  32,  71, 108, 101, 105, 116, 107, 111, 109, 109,  97,  98, 105,  98, 108, 
105, 111, 116, 104, 101, 107,  32, 109, 105, 116,  32,  90, 117, 102,  97, 108, 108, 115, 122,  97, 
104, 108, 101, 110,  46,  13,  10,  13,  10,   0, 105,  32,  61,  32,   0,  35,  35,  32,  49,   0, 
 35,  35,  32,  50,   0,  35,  35,  32,  51,   0,  35,  35,  32,  52,   0,  35,  35,  32,  53,   0, 
 35,  35,  32,  54,   0,  35,  35,  32,  55,   0,  35,  35,  32,  56,   0,  35,  35,  32,  57,   0, 
 35,  35,  32,  49,  48,   0,  35,  35,  32,  49,  49,   0,  35,  35,  32,  49,  50,   0,  35,  35, 
 32,  49,  51,   0,  35,  35,  32,  49,  52,   0,  35,  35,  32,  49,  53,   0,  35,  35,  32,  49, 
 54,   0,  35,  35,  32,  49,  55,   0,  35,  35,  32,  49,  56,   0,  35,  35,  32,  49,  57,   0, 
 45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  70,  69,  82,  84,  73,  71,  45,  45, 
 45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,   0,  42,  42, 
 42,  42,  42,  42,  42,  42,  42,  42,  42,  32,  70,  69,  72,  76,  69,  82,  32,  42,  42,  42, 
 42,  42,  42,  42,  42,  42,  42,  42,  42,  42,   0, 101, 114, 114, 110, 114,  32,  61,  32,   0, 
114, 110, 100, 115, 101, 101, 100,  95, 120,  32,  61,  32,   0, 114, 110, 100, 115, 101, 101, 100, 
 95, 121,  32,  61,  32,   0, 114, 110, 100, 115, 101, 101, 100,  95, 122,  32,  61,  32,   0, 114, 
110, 100, 115, 101, 101, 100,  95, 119,  32,  61,  32,   0,  78,  97,  78,   0,  73,  78,  70,   0, 
 65, 242,  47, 244,  37,   0,   0,   0, 241, 135, 147, 231,  18,   1,   0,   0, 243,  15, 167,  37, 
165,  11,   0,   0, 203, 145,  13, 206, 247, 103,   0,   0, 145,  88,  92,  99,  58,  64,   3,   0, 
238, 213,  40,  65, 106, 193,  22,   0, 237, 134,   8, 225, 136, 136, 136,   0,  62,  29,  37, 160, 
170, 170, 170,   2, 192, 161, 102, 171, 170, 170, 170,  10,  77,  67, 249, 255, 255, 255, 255,  31, 
 11,  24,   0,   0,   0,   0,   0,  64, 242, 255, 255, 255, 255, 255, 255,  63, 192,  89, 148, 178, 
  6,   3,   2,   0, 106, 109,  18, 116, 169, 200,  89,   0, 228, 184,  45,  12,  38,  40, 246,   3, 
165, 132, 149, 216, 210, 165, 223,  17, 204,  45, 218, 250, 244, 241,  92,  37, 223, 107,  74, 125, 
146,  72,  51,  36, 229, 145,  14,  33, 113, 233,  34,  13, 128,  24, 156, 173,  48,  94,   0,   0, 
168, 133, 140, 208, 242, 104,  26,   0, 126, 254,  71, 231,  83, 153, 176,   1, 238, 104,   0,  73, 
149,  75, 242,  10, 219,  39,  71, 130, 208, 247,  21,  34, 142,  39, 190, 118,  31,  88,  94,  54, 
 60, 181, 209,  13,  75, 189, 196,  42, 230, 145,  14,  33, 113, 233,  34,  13,  21, 129, 176,  43, 
  0,   0,   0,   0,  57,  74,  29,  26, 229, 255, 255, 255, 114, 249, 168, 241, 142,  11,   0,   0, 
 74, 218, 200, 252, 203, 191, 252, 255,  83, 152, 135, 136, 136, 136, 136,   0, 177,  94,  85,  85, 
 85,  85,  85, 245, 240, 255, 255, 255, 255, 255, 255,  63, 231, 113, 222, 144, 245,  81,   0,   0, 
193, 205, 208, 201,  32,  33,  37,   0,  80, 194, 255, 129, 121, 137, 119,   2,  16, 226, 191, 179, 
 91, 162,  69,  14, 173,   8, 118, 146,  30, 174,  96,  35, 136,  25, 208, 194,  94, 252, 211,  38, 
200, 219,  34, 200, 130,  49, 127,  15, 217, 218,  75, 155, 131,  71,   3,   0,  77, 107, 142, 224, 
181, 231, 130,   0, 157, 180,  46,   6, 210, 103, 107,   5, 229,  25, 170, 100, 110, 176,  85,  23, 
152,  91, 108, 142, 141,  94, 241,  46, 174,  86, 134,   5,  53,  98, 254,  43, 206, 219,  34, 200, 
130,  49, 127,  15,  14,   0,   4,   0,   8,   0,  17,   0,   0,   0,  35,   0,   3,   0,  12,   0, 
  5,   0,  50,   0,  49,   0,  54,   0,  41,   0,   1,   0,  48,   0,  42,   0,  27,   0,  55,   0, 
 56,   0,   2,   0,  39,   0,  28,   0,   6,   0,  26,   0,  31,   0,  63,   0,  53,   0, 233,  55, 
 43,   0,  23,   0,  59,   0,  15,   0,  37,   0,  60,   0,  45,   0,  44,   0,  34,   0,  13,   0, 
 16,   0,  38,   0,  43,  56,  22,   0,  66,   0,  24,   0,   7,   0,  21,   0, 184,  56,  67,   0, 
130,  57, 203,  56,  71,   0,  47,   0,  11,   0, 108,  57,  51,  56,  75,   0,  70,   0,  46,   0, 
 20,   0, 209,  56,  83,  57,  71,  56,  57,   0,  27,  57,  19,  56,  67,  57,  20,  57,  65,  56, 
 35,  56, 122,  57,  58,  57, 237,  56, 229,  56, 222,  56, 114,  57,   3,  57, 245,  56, 215,  56, 
177,  56,  58,  56,  69,   0,   9,   0,  89,  57,  76,  57,  58,   0,  97,  57,  33,  57, 191,  56, 
 25,  56, 239,  55,  51,   0,  40,   0,  33,   0, 193,  57, 170,  57, 158,  57, 135,  57,  44,  57, 
  9,  57,  65,   0,  61,   0,  18,   0, 185,  57, 176,  57, 164,  57, 151,  57, 145,  57, 253,  56, 
160,  56,  76,  56, 250,  55,  68,   0,  64,   0,  62,   0,  29,   0,  19,   0,  73,   0,  36,   0, 
 30,   0, 208,  14,  90,  23, 234,  22, 222,  20, 230,  23, 198,  21, 144,  19, 250,  23, 126,  27, 
144,  32, 192,  14,  50,  50, 152,  27,  92,  45,  64,  42,  74,  21, 176,  43, 186,  52, 244,  25, 
222,  19, 250,  28, 228,  31,  88,  27,   8,  26, 238,  29, 228,  18, 192,  19,  96,  21,  17,  51, 
  3,  32,  49,   6, 255,  51,  69, 177,  49,  18,  64, 144,  22,   2,  50, 227,  45,  49,  70,  10, 
 64, 134,  64,  99, 243,  63,  47, 245, 255,   7,  79, 240,  53, 255,   3, 111,  80,  53, 254,   3, 
 99,  48,  70, 253,   3,  95, 255, 245, 253,   3, 111, 240,  54, 244,   3,  91, 119, 133,  80,  70, 
 81, 224,  84, 255,   7,  79, 224,  52, 221,   3,  75, 208,  55, 255, 255, 255, 255, 111, 255, 240, 
  0,   6, 221, 188,  27,  79, 108, 208, 208,  16,   9, 255, 255, 255, 255, 255, 255, 255, 127, 144, 
  0,   0,   0,   0,   0,   0,   8,   9,   0,   0,   0,   0,   0,   0, 240, 127, 159, 255, 255, 255, 
255, 255, 255, 255, 249,   0,   0,   0,   0,   0,   0,   0, 160, 159, 255, 255, 255, 255, 255, 240, 
240,   9,   0,   0,   0,   0,   0,   0, 240, 255, 144,   0,   0,   0,   0,   0,   0,  15, 249,  26, 
 97, 180,  16,  81, 237, 135, 100, 144,   0,   0,   0,   0,   0,  14,  15, 249,  53, 194, 104,  33, 
162, 218,  15, 201, 154, 199, 156, 253,  31, 113, 119,  43,  25,  35, 219, 249, 126, 106, 220,  94, 
 64, 145, 130, 212,  69,  79, 178,  31, 147, 249,   0,   0,   0,   0,   0,   0, 240,  63, 152, 211, 
  5, 168, 138, 143, 100,  51,  32,   0,   8, 101,   3, 209,  64,  12, 248, 129,  14, 105,  64, 188, 
144, 139,   2,  12,  73, 186, 105,  64,  94, 255,  43,   2,  12,  73, 186,  64, 196,   9, 207, 101, 
  3, 230,  64,  12, 248, 129,  14, 105,  64, 124, 129, 198, 167, 239, 157, 178, 105,  64,  94, 241, 
186,  94,  53,  63, 125,  65,  68,   9, 207, 101,   3, 250,  64,  12, 248, 129,  14, 129,  55,  64, 
180,   9, 207, 136,  16, 225,  55,  40, 200,  64, 156, 254, 128,   1, 144, 135,   6,  38,  74, 194, 
 63,  13,  64,  97,  48, 198,  64,  97,  39, 198, 203, 114, 175, 248,  17, 123, 203, 145,   6, 210, 
242,  64,  97,  44, 198, 242, 254,  12, 237, 208,  44,  70,  80,  64, 132,   0, 207, 135,   6,  38, 
 74, 194,  19,  13, 255,  84,  12,  80, 253, 208,  44,  70,  80,  67, 132,   0, 207, 101,   4,  83, 
 64,  12, 248, 112,  98, 100, 172,  33,  48, 223, 245,  65,  69,  15, 221,   2, 196, 101,   4, 129, 
 64,  12, 246,  80,  73, 196,   0, 207, 135,   6,  35,  42, 194,  22,  13, 143, 224, 201,  64, 181, 
 15, 221,   2, 196, 101,   4, 202,  64,  12, 246,  80,  78,  84,   0, 207,  72, 119, 172,  26,   3, 
  0,   3,   0, 255, 153, 193,   9,   3,   0,   3,   0, 255, 154, 193,   5,   3,   0,   6, 179,  30, 
 16,   0, 201, 255, 160,  31, 225, 194,   6,  29, 107,  49, 193,   0,  58, 170, 169,   2, 136, 232, 
 56,  23, 169,   8, 216, 119, 196,   7, 113,  39, 172,  47, 240,  53,   0, 208, 119,  18,  36,   0, 
140, 130,  28, 149, 135, 244,  89, 114, 156, 149, 167,  45, 201,  91, 116,  25, 216,  63,  64, 165, 
 15,  65, 157, 132, 244,  10,  81, 246,  80,  82,  52,   0, 253, 251, 248,  64, 196,  10, 156,   2, 
  2,  29, 204, 127, 240,  95,  38,  80,  82, 132,   0, 253, 252,  95, 132, 142,   0,  64, 169, 192, 
 32,  34, 220, 199, 255,   5, 160, 101,   5,  45,  64,  15, 223, 213, 248,  72, 225, 228,  10, 156, 
  2,   2,  61, 204, 127, 240,  84, 230,  80,  83,  36,   0, 253, 255, 235, 248,  65,  68,  10, 156, 
  2,   2,  77, 204, 127, 240,  79, 230,  80,  83, 116,   0, 234,  37, 253, 100, 151,  70, 156,   2, 
  2,  93, 204, 127, 240,  75,  38,  80,  83, 196,   0, 234,  38, 253, 100, 151, 152, 156,   2,   2, 
 93, 204, 127, 240,  70, 102,  80,  84,  20,   0, 234,  39, 253, 100, 152,  58, 156,   2,   2,  93, 
204, 127, 240,  65, 166,  80,  84, 100,   0, 234,  40, 200, 128,  63,  64, 185, 192,  32,  37, 220, 
199, 255,   3, 204, 101,   5,  75,  64,  14, 162, 156, 130,  16,  63,  64, 185, 192,  32,  38, 220, 
199, 255,   3, 124, 101,   5,  80,  64,  14, 162, 172, 130,  32,  63,  64, 185, 192,  32,  39, 220, 
199, 255,   3,  44, 101,   5,  86,  64,  14, 162, 188, 136,   3, 244,  17, 156,   2,   2, 141, 204, 
127, 240,  45, 230,  80,  85, 196,   0, 234,  44, 200,  33,   3, 244,  17, 156,   2,   2, 157, 204, 
127, 240,  40, 230,  80,  86,  36,   0, 234,  45, 200,  34,   3, 244,  17, 156,   2,   2, 173, 204, 
127, 240,  35, 230,  80,  86, 132,   0, 234,  46, 253, 100,  18, 156,   2,   2, 173, 204, 127, 240, 
 31, 102,  80,  86, 228,   0, 234,  47, 253, 100, 142, 210, 156,   2,   2, 173, 204, 127, 240,  26, 
166,  80,  87,  68,   0, 234,  16,  79, 214,  65, 137, 192,  32,  43, 220, 199, 255,   1,  96, 101, 
  5, 122,  64,  12, 255, 134,  49,  44, 143, 132, 143,  56, 156,   2,   2, 205, 204, 127, 240,  16, 
166,  80,  88,   4,   0, 207,  53, 255, 134,  49,  60, 131, 111, 248,  73, 105, 137, 215,  11,   5, 
240, 111, 156,   1, 130, 221, 204, 121,  46, 133, 193,  96, 101,   5, 134,  64,  12, 240, 191,  16, 
140, 128, 191,  72, 205,  73, 192,  24,  46, 220, 199,  52, 209,   7, 118,  80,  88, 196,   0, 207, 
 36, 200, 193, 101,   5, 178,  64,  12, 240, 119,  18,  36,   0, 140, 135,   6,  80,  93,  52,   0, 
140, 128, 135, 101,   5, 220,  64,   8, 200,   9, 118,  80,  94, 148,   0, 140, 128, 167, 101,   5, 
246,  64,   8, 200,  11, 118,  80,  96,  52,   0, 140, 129,   0, 194,   6,   5, 193,   1,  20,   0, 
  4,   6,  64, 102,  55, 252, 108,  49,  44, 169, 193,  64,  98, 252, 109, 128,  54,  32, 234, 231, 
  6,  31,  64, 111, 124,  33, 146, 161,  23,  64,  97,  21, 198,   0,  63,  21,   3,  17, 116,   6, 
 17,  92,  96,   3, 205, 197,  80,  52,   6, 247,  17,  61,  32,   3, 203, 145,  21, 198,  16, 109, 
 47,  39,  17,  48, 198, 242, 193,   0,   0,   2,  18,  28, 226,  34,  92, 226,  66, 156, 226, 130, 
220, 236,  16,   1,  25,   0,  15, 244,  33, 255,  60, 185,  16,  61,  37,  15, 255,  67,  15,  33, 
255,  63, 213,   0, 252,  16,  85,   4,   0,  14,  16,   4, 201, 225,   0,  15, 225, 194,   6,  29, 
193,   5,   3,   0,  12, 165, 225,   0,  12, 156,  32, 160, 221, 208,  39,  12,  21,   3,  22,   0, 
 12, 161,  16, 221,  48,   9, 195,  28,  33, 204,  13,  16,   0, 211,   0, 194,  30,   7, 225,   0, 
 12, 150, 195,  11,   0, 113,  15, 213, 181,   7, 112, 192,  24,   0, 125, 245,   7,   0, 124,  16, 
 86,   4,   0,  16,   0, 226, 192,  44,  16, 222,  16,   8, 206,   0,  13, 245,   0,   0, 142, 128, 
  1,  48,  16,  32,   0,  42, 255, 139, 255, 160,  28, 227,   0,  42, 255, 188,  64,   0, 195,  63, 
143, 227,  15, 236, 128, 201,  80,  29,  48, 143, 227,   0,   1, 212,   8, 112,   0, 137, 208,  12, 
112, 238,   0, 137, 195,  85, 193,   3,  85,  85,  10,   0,  13,  48,   1,  48, 198, 212,  12, 211, 
  0, 253, 239, 252,   8, 194,  42,  10,  17,  93,  48, 137, 192,  56,   0, 198, 144,   1,   0,   0, 
  0,   0,   0,   0,  15, 213, 141,  64, 192,  12,   1,  14,  93,  48,   1,  19, 205, 247, 212,   4, 
193,   4,  85,  85,  10,   0,   8,   0, 192,   8,   0,  64,   0,  64,  61,  48, 141,   0, 193,  21, 
211,   8, 156,  10,   1,  26, 211,   8, 197,   1,  12,  85,  16,  98, 192,   1,  10, 208,  11,   1, 
  1,   9, 172,  34,  48, 160,  16, 226, 194,  16,  16,   0, 192,  16, 223, 229,   0, 192,  16, 239, 
141,  64, 204,  16,  84,  53,   8,   0,   0,  10, 217,  15, 227,   8, 217,  23,  12,  44,  49, 231, 
 38, 159, 255, 254,   0,   0,   0,   0,   0,  12, 110, 113,  34, 189, 164,  40, 229, 209, 240, 103, 
 33,  49, 198, 231,  18,  13, 218,  66,  30,  93,  24,   6, 114,  19,  44, 108,  33, 224, 215,  66, 
140, 108, 177, 204,  93, 164,  36, 239, 209,  48, 103,  33,  52, 198, 194,  28,  13, 116, 247, 203, 
 28, 197, 218,  66,  30, 253,  16,   6, 115, 192,  62, 221,   2, 114,  19,  76, 108,   2,  13, 162, 
 33, 239, 209,   0, 108, 162,   0, 106, 195,  24, 133,   6, 140, 194,   0,  97,  35, 172,   1,   8, 
 80,  96,   6,  18,  58, 195,  28,  17,  85,   6, 140, 192,   0,  76, 185,  17,  61,  32,   6, 203, 
145,   6, 210, 242, 113,  19,  12, 111,  45,  64,  12,  21,   3,   0, 140, 162, 232,   0,  25,   1, 
 50, 216,  15, 196,   0, 253,  48,  12, 109,  96, 167,   1,   9, 172,  33,  18,  80,   0,  40, 229, 
221, 114,  18, 205, 128, 252,  64,  15, 211,   0, 198, 214,  10, 112,  16, 154, 194,  17,  37,   0, 
  2,  30,  93, 208,  39,  12,  19,  85,   7,   0,  13,  48,  15, 212, 211,   4, 253,  77,  93,  48, 
 15, 212, 211,   4,  18, 124, 109,  81,   3, 241, 211,   4, 253,  77,  48,   1,  39, 198, 213,  16, 
 63,  16,   8, 253,  13,   0, 221,  48,  12, 184, 226, 192,  26, 218,  13,  48,  78,  48,   8,  16, 
188, 109,   0, 221,  48,  76, 184, 226, 192,  26, 218,  13,  48,  14,  55,  12,  19, 149,   7,   0, 
  0,  20,  48,  64,   0,  64,  28,  21,   3,   1, 189,  48,  14, 128,   3, 153,  80, 112, 253,  76, 
183, 232,  18,  55, 115,   7,  28, 183, 103, 128,   0,   0,   0, 198, 237,   9, 254, 192,  12,  42, 
176, 135,  33,   4, 205,  82,  48,  35, 224, 183, 255, 252, 108,  38, 212,  65,  28,   2,  57, 194, 
 89,  14,  33,  18, 112,  35,  33, 205, 197,   2,  63, 235, 188, 181,  82,  48,  35, 203, 113,   8, 
210, 119,   2,  48,  35, 203, 118, 112,   0,  32,   0,  11, 122, 203, 157,  93,  81,   3, 241, 197, 
 33, 197,  82, 183,  77, 245,  43, 114,  17, 204, 108,  49, 104, 116, 197, 193, 114,  17, 204,  98, 
 45,  45, 245,  43, 133,  43, 105,   0,   0,   0,   1,   0,   0,   0,   0,   2, 188, 185, 242, 113, 
213, 113,  33, 241, 216,  31, 216,  51, 196,   1, 255, 212, 216,  53, 196,   3,  92,  34,  80,  80, 
 51, 115, 176,  51, 172,   1,  45,  16,  53,  51,  55,  61, 237,  16,  43,  18, 192,  43, 203, 150, 
 49, 253,  47,  37,  15, 128,  15, 197, 135,  50,  29,  35,  15,  64,  76,  85,  31, 131,  31,  48, 
244,   1,   1, 241,  44, 198, 195,  22,   0, 252, 112, 236, 169, 181,  15,   0, 242,  27,  33, 205, 
193,   0,   6,   3,  25,   0,   0,   0, 226, 192,  26,   0,  13, 252, 112, 224,   0, 203, 145,  14, 
210, 218,   6,  80,  65, 240,   0, 226, 192,  22,  16,  12, 112, 236, 166,  48, 244,   5,   0, 252, 
 25,   3,   0,   0,   0,  18, 188, 108,  21,  57, 149,  10,  83,  68, 153,   0,   2, 163,  25, 252, 
 48,  68,   3,  42,  50, 147,   9,  48,  19,  20,  64,  56, 212,  38,  17,  80,   5, 156,  33, 166, 
 65,  21,   0, 153, 195,  26,   1,  77,  64,  12,  16,  25, 214,  25,  17,  80,   9, 157, 112, 224, 
 41, 214,   7,   2,  76,  32, 208, 224,   4, 212,   0,  18, 189,  64,  12,  16,   5, 208,  11, 251, 
 17,  27, 172,  50,  96,  20, 212,   0,  33, 212,  38, 193,   0, 157,   0, 207, 211,  17,  27, 172, 
 49, 160,   4, 212,   0, 193,   2,  76,  35,  56, 207, 211, 172,  33, 177, 192,  41,   1, 159, 188, 
 95,  21,  25,   0, 149,   5,  49, 144,  41, 253,  60,  95, 242, 251, 173,  97,  63, 211, 157, 112, 
176,  25,   2, 154, 194,  34,  28,   2, 144,  25, 251, 253,  44,  85,  25,  33, 212,  38,   0, 149, 
  5,  49, 144,  41, 253,  63, 210, 227,  49, 144,   5,  16, 252,  88,   0,   4,   5, 193,  37,   4, 
  0,   0,   4, 194,  15,  22,   0,   1,  44, 253,  80,   0,  18, 191, 246, 196, 193,   9,   3,   0, 
  3,   0, 217,   1,  19, 205, 247, 203,  60,  16, 153,   5, 163,  25,   0,   3,  23,   0,   4,   8, 
 80,  16,  16,  64, 133,   3, 254, 156,  34,  66,  99,   7, 128,  16,   0,   7, 100,  23,   0, 112, 
  1, 253, 191, 208,  80,  83,   7, 246,   0,   7,  68,  23,   0,  21,   5,   0,  87,  20,  22,   1, 
124,  16, 153,   5,  19,   0, 128,  16, 232,   1,  25,   7,   4,   8,  80,  19,   1,  33, 243,   0, 
 29, 161,  65, 103,   0,   0,  64, 197,  17,   1,  28,  16, 153,   5,  99,  52,  73, 144,   0,  43, 
253, 212,   3,  43, 254, 132,   3, 250, 194,  98,  64, 245, 194,  59,  62,   0, 125,   0, 112,  11, 
194,  38,  10,   0, 112,  11, 190,  11,   4,   9, 197,  80, 120,  50, 179,  27,  64,  21,  27, 202, 
156,  16,  43, 214,   7,   0, 124,  49,  33,  43, 193,   0, 181,   7,   2, 181,  27,   1, 189, 112, 
223,  93,  96, 112,  43, 214,   7,   0, 188,  49,  33,  43, 193,  48,  16,   3, 243,  49, 176,   7, 
  0,  19,   0,  64,  80,   0, 193, 153,   5, 115,  52,  73, 153,   0,   2, 191, 221,  64,  50, 191, 
232,  64,  48,  11, 194, 105,  10, 250, 194,  60,  74, 245, 194,  35,  34, 131,  43,  64, 115,  27, 
 64,  69,  59,  48, 113,  30,   0, 188,  93, 224,  43, 195,  18, 202, 156,  17,  43, 193,   1, 189, 
 96, 111,  92,  49,  33,  43, 193, 254, 156,   1,  97,  49, 199,  14,  18, 220,  17,  43, 193,  51, 
176,   7,  16, 252,  95, 233,  48,   4,   5,   0,  12,  16,   9,   3,   0,   0,   0, 253, 236, 180, 
112, 208,  13, 112, 100,   4,  51, 172,  49,  32,   0, 193, 112,  17, 106, 192,  18, 202, 156,  16, 
  0,  18, 225,  26, 115, 197, 210, 198, 193,   9,   3,  51,  73,   0,   8,  49,  95, 218,  64, 240, 
  5, 100,   4,  31, 172,  49, 108, 166, 199,  58,   0,  28,   2,   0,  21, 203, 109, 252, 113,  64, 
 21, 203, 108,  25,   3,   0,   1,  45,   0,   1,  45, 198, 157,  96, 192,   0,  19,  12, 108, 177, 
203,  60,  25,   3,   0,   1,  45,   0,   1,  45, 198, 156, 179, 193, 153,   5,  99,  52,  73, 144, 
  0, 143, 221,  64,  56, 254, 132,   3, 250, 214,   7,   1, 189, 113,  15,  93,  96, 112,  43, 203, 
 28,   1,  44, 162, 193, 250, 195, 148, 245, 194,  28,  80,   0,  28,   1,  97,   5, 199,  14,  16, 
 12, 178, 193,   0,  16,   3, 156,  49, 108, 166, 199,  42,   0,  28,   1,  97,   5, 199,  14,  16, 
 12, 178, 193, 245, 195,  52,   0,  60,   1,  97,   0, 199,  14,  16,  92, 178, 193,   0, 124,  53, 
224,  11, 215,   7,   2, 188,  49, 108, 166, 199,  42,   0,  60,   1,  97,   0, 199,  14,  16,  92, 
178, 193,   0, 190, 208, 176,   1,   0,  57, 215,  11,   0, 176,   7, 172,   3,  64,   1, 192,  22, 
 16,  92, 112, 225,   0, 203,  44,  16,   7,   0, 186, 192,  52,   0,  28,   1,  97,   0, 199,  14, 
 16,  92, 178, 193,   1, 176,  43, 156,  49, 108, 166, 199, 106,   2, 176,  27, 174, 208, 112,   1, 
237,  21,   1, 176,  43, 173,   0, 112,   1, 203,  28,   1,  97,   0, 199,  14,  16,  92, 178, 193, 
 88,   5,   0,   1,  47, 202,  78,   4,  56,  18, 205,  65,   0,   0, 226, 192,  34,  33, 204,   0, 
  0, 223,  80,   0,   0, 194,  85,   5,   0,  15, 124,   4, 104, 218,  96,  16,  64,  29,  65,  13, 
164, 116, 240,   1,   4,   2, 197, 222, 141, 166, 218, 116,   1, 204,  87,  50,  29,  47,  13, 166, 
 64,  44,  92, 195,  48,   2,  30, 247,  12,   3, 160,  16,  64, 125,  65,   7,  45, 240,  16,  64, 
 44,  92, 194, 114, 193,  83,  57,   9, 163,  68,  73, 144,   0,  98,  17,   1,  10, 192,  20,  16, 
133,  16,   1,   6,  32,  26, 192,  18,  33,  81,   2, 179,  31,  48, 191, 217,  64,  48,  11, 195, 
 48,  16,  33,   1, 206, 134, 177,  44, 225,   1, 193, 245, 195, 140,   1, 252,   2, 166,  80,  97, 
  1,   8,  65, 161,   1, 193,   0,  28,   1, 134, 194, 220, 113,   6, 194, 177,   1, 206, 101,   6, 
 20,  99,  18,  65, 161,   1, 193,  48, 177,  23, 227, 133,   3,   0,  28,   1,  65,  13, 253, 192, 
 11, 226, 194,  29,  52,   0, 189, 252, 181, 224, 105, 213,  37, 205, 223,  80, 240,  11, 203,  81, 
 15, 213, 108,  31, 178,  92, 220, 181,  80, 243,  11,  50, 240,  15, 223,  65,  93, 232,  50, 243, 
 31,  64,  21,  31,  48, 177,   0,  49, 244,   2, 197, 222,   0, 190,  44,  34, 144, 152,  18, 243, 
 31,  64,  69,  31,  48, 190,   6,  67,  31,  64,  44,  93, 237,  24,  15,   0, 182,  32,  74, 208, 
 41,   0, 178,  57, 215,  31,   1, 246, 159,   0,   0,   0,   0,   0,   0,   0,  12,  97,  47, 174, 
115, 118,  24, 105, 204, 204, 204, 204, 204, 204, 204, 204,  49, 244,   4,  81, 243,  11,  49, 244, 
  2, 224,  99, 190,  61,  16,  15,   0,  53,   5, 209,   0,  51,  16,  33, 254, 162,  27, 194,  97, 
 21,  16,  32,   3, 233, 206,   0, 190,  44,  53,  32,   3, 233,   1, 241,  18,   0, 188,  95, 242, 
 49, 241,   0,   0, 187, 229,  17,  69,  11, 209,   0,  56,  18, 243,  31,  64,  69,  31,  48, 190, 
  6,  67,  31,  64,  44,  93, 224,  11, 226, 215,  21,   1, 241,  18,   0, 191, 210,  98,   5, 172, 
 63,  96,   3, 216,   7, 196,   0,  94, 232,   7, 172,  34, 241,  16,   7, 233, 201,  99,  57, 156, 
 33,  49,  64,   7, 233,  33, 222, 253, 129,  16,  32,   7, 233, 206,   0, 112,   5, 156,  53, 237, 
 16,  15,  99,  49, 238,   0, 126, 156, 238, 224,   7,   0,  58, 194,  16,  17,  16,  32,   7, 233, 
206,   0,  95,  12, 181, 233, 201,   0,  94, 156, 225,  12,   0,  95,  12, 181, 233, 206,   1,  44, 
  1,   8, 253, 198,  52,  95, 220, 202,  32,  15, 172,   1, 102,  50, 191, 220,   0,  62, 144,  15, 
 72, 162, 160,  20, 192,  22,   0, 253,  65,  65,   1, 193,  51, 144, 113,  64,   2,   0,   1,  44, 
198, 192,  32,   1,   1,   0, 197, 199,  20,   1,  12, 182, 203,  55,  15,  12, 179, 202,  94, 128, 
 34,  51,   8, 253,  16,   5,  98,   3, 173,   0, 151,  39,  41, 194, 184,  33, 115, 117, 172,   3, 
  3,   3,  33, 117,   0,   4,   9, 204,  47, 216,   3,   3,  33, 118,   0,   4,   9, 204,  32,   0, 
253, 225,  21, 157,   0, 192,   0,  18, 220, 108,  49,   7,  44,  16,   3,   1,  45, 241,   0, 197, 
172,   1,  15, 216,   7,  76, 195,   0,   4,   8, 192,  32,   1,   1,   0, 197, 199,  20,   1,  12, 
182, 204,  64,   3, 226, 194,  19,  32, 218,  96,   3,  16,  12,  93, 224,   3, 240, 117, 172,   1, 
 13, 216,  77,  16,   5,   0,   1,  45, 198, 237,  13,   0,  48,  18, 223, 173,  96, 160,   3, 115, 
172,  47, 240,  24,  79, 240,  34,  64,   0,  64, 140,   1,  13, 208,  32,   3, 226, 194, 158, 255, 
  1, 110, 116,  65,  63, 209, 202,  46,  24,   5, 174, 208, 193,   2, 249, 201, 158, 112, 242, 152, 
  0,  87, 139, 206, 217,  71,  82,  27, 206, 238,   0,  92,  76, 162,   0,  90, 194,  32,  13,   0, 
 87, 123, 201,   0,  80,   3, 197, 120, 188, 237,  24,   5,  16,  47, 227,  65,  12, 117,  33, 188, 
226,  47, 209, 238,   0,  62,  44,  33,  48, 209,   2, 225,   0,  87, 139, 206,  33, 253,  16,   3, 
  0,  90, 231,  68,  13,   0,  95,  12, 181, 117, 188, 159, 159, 225, 195,  78,   0,  48,   5, 174, 
113,  48, 209,   2, 225,   0,  87, 139, 206, 143, 156, 239, 216,  13,  16,   5,   0,  95,   0,  16, 
173,   0, 176,   5, 115, 228, 194,  74,  18,  16, 207, 156, 231,  68,  19, 253,  28, 162, 225, 128, 
 90, 237,  12,  16,  47, 156, 153, 231,  15,  41, 128,   5, 120, 188, 225,  12, 249, 201, 156,  49, 
 40, 249, 206, 128,   5, 116, 188, 224,   0,  64, 140,   1,  13, 216,  32,  16, 224, 100, 197,  80, 
 80,   0,  64, 140,   1,  45,  24,   5,   0,  15, 222,  80,  48,   3,  17, 106, 192,  18, 209, 128, 
 86,  64,  66,  16,   3, 173, 112, 176,   3,  18,  90, 192,  18, 209, 128,  80,   5,  98,   1, 172, 
  1,  34,  21,   5, 136,   0,  80,   0,  64, 159, 253,  12,  64,  16, 113,  65,  58, 194,  24,  51, 
238, 128,  86,  32,  26, 192,  16, 253, 128,  34, 253,  16,   5, 113, 228, 208,  11,  99,  69, 254, 
125,   0, 182,  54,  95, 231, 194,  11,  57, 209,   0,  80,   5,  80, 126, 232,   5,  98,   4, 173, 
  1,  96,   0,  64, 141, 112, 192,   5,  98,   3, 173, 112, 161,   2, 254, 126, 112, 101,  62,  16, 
  7, 113, 228, 238,   0,  87,  43, 254,  28,  32,  98, 119,  12,  21,  80,  80,  28,   0,   8, 142, 
128,   5,  73, 148, 144, 217,  46, 209,  93, 146,  16,  57, 208,  10, 217,  34, 153, 231,  10,  47, 
221,   4, 217,  38,  54, 233, 208,  12, 217,  38,  52, 233, 214,  33, 114,  33, 228,  99,  97, 157, 
  0, 247,  34,  30,  70,  52,  25, 254,  45,   2,  23,  34,  46,  70,  54, 233, 208,  15, 114,  34, 
228,  99,  78, 159, 226, 192,  32, 221,  34, 199, 255,   4,  40, 217,  33,  13, 156,  33,  97,  45, 
148,  99,  43, 156,  49,  13, 208,  66,  28, 193, 221,   2, 217,  38,  54, 153, 208,  12, 217,  38, 
 52, 153, 214,  33, 114,  33, 228,  99, 110, 157,   0, 247,  34,  30,  70,  52, 233, 254,  45,   2, 
 23,  34,  46,  70,  54, 105, 208,  15, 114,  34, 228,  99,  70, 159, 226, 192,  86,   0,  76,   2, 
 78,   6,  60, 181, 115, 189,  64,  71,  28,   1,  97,  49, 199,  14,  18, 220,  24,  82, 120,  81, 
 50,  53,   3,  16,  12,  43,  64,  93, 148,  16, 201, 195,  60, 221,   4, 114, 192,  30, 255,  28, 
127, 240,  38, 226,  28, 194, 217,  70,  35,  10, 215,  12,  98,  57, 217,  90, 192,  16, 253, 128, 
 98,  59,   0,  58, 194,  17, 160, 116, 195,  18, 209,   2, 119,  44,   1,  45,  24,  39, 129,  47, 
 49,  52,   4,  81,  48,  19, 192,  18, 209,  48,  51,  19, 217,  92, 185,  16,  44,  81,  18,   0, 
 60,  93,  45, 224,  19, 192,  30,  48,  51,  19,  64,  46,  61, 208,  77, 146,  99,  69, 157,   0, 
205, 146,  99, 101, 156,  57, 131,  39, 254, 192,  52, 137, 254, 222, 217,  33,  13, 157,   0, 205, 
146,  99,  43, 156,  49,  13, 208,  45, 146,  98,  48, 173, 112, 198,  35, 157, 147, 174, 112, 162, 
125, 208,  67,   3,  50,  48,  39,  65,  95,  13, 232,  50,  51,  19,  64,  21,  19,  49,  49,  31, 
  0,  59, 115,  51, 116,   5,   0,  76,   1,  71,  45,  64,  64,  55, 193,   0,  76,   1,  71,  45, 
 64,  65,  43, 193,   3,  83,  57,  11,  57, 147,   0,   0,  16, 194, 100,  42, 141, 128, 252,  64, 
 24,  16, 188, 110, 208, 160,  16, 253,  12,   2,  67,  15, 211,  20, 255, 124, 113, 195,  15, 211, 
 20, 254, 172,  78,  23,  20, 196, 238, 129,  12, 177, 194,  24, 125,   1, 143, 208,  48,   3,  15, 
 64,  21,  15, 105,   8,   0,   0,   0,   0,   0,   0,   0,  80, 240,  18, 195,  18,   0, 252,  24, 
216,  31, 196,   1, 129,  11, 198, 237,  10,   1,  47, 208, 192,  36,  49, 253,  49,  79, 247, 199, 
 28,  49, 253,  49,  79, 234, 196, 225, 113,  76,  78, 232,  18, 203,  28,  33, 135, 208,  24, 253, 
  3,   0,  49, 244,   1,  81, 248,  82,  17,  50,   1, 252, 110, 114, 112, 243,  31,  40, 229,  19, 
 32,  15, 198, 194,  11,  14, 209, 114,  19,  15,  40, 229,  18, 192,  31, 198, 231,  39,  15,  49, 
242,  30,  81,  44,   0, 252, 108,  32, 176, 237,  16,  33,  48, 242,  30,  83,  31,  64, 117,  31, 
131,  31,  48, 244,   1,   2,  29,  44,  21,  84,  57,  11, 115,  52,  73, 153,   0,   2, 191, 221, 
 64,  48,  26, 192,  18, 141,  65, 160,   7, 195,  18, 202, 156,  31, 173,   0, 177,  21,   1,  41, 
208,  13,   1,  45,  49, 111,  44,  50, 224,  26, 192,  22,  18, 189,  65, 161,  43, 193,  42, 216, 
  3, 196,   0, 112,  27, 242, 237,  34,   1,  32,   7, 173,  97, 112,   7,   1,  41, 215,  12,   1, 
189,  49, 106, 214,   9, 238, 128,  60,  43, 118,  88,   1, 100,   7,  37, 197,  33, 210,  49, 180, 
  4,  37, 197,  83, 176,   7,   1,  41, 215,  12,  18, 192,  59, 198, 195,  20,  18, 197,  59,   0, 
112,  18, 197,  80, 182,  35, 240,  11, 172,  51,  35,  59,  18, 225,  18,   0, 188,  93,  47, 246, 
196,  49, 184,   1,  99,  59,  64,  18,  29,  46,  48,  26, 192,  86,  51, 177,  31,   0, 187,   0, 
 16,  16, 253, 179,  43,  64,  80,  43, 211,  26,  64, 205,  65, 163,   7,  49, 180,   2, 227,  49, 
176,   7, 224, 107, 197,   0,  19,   0,  64,  80,  27, 212,  22,   0,  60,  49,  44, 169, 193,   0, 
 12,  21, 153,   7,  51,  73,   0,   2, 179,  21, 252,  49,   4,   3,   2,   3,  21,   0,  80,   1, 
  0,   4,  16, 193, 153,   5,   1,  28, 169, 202, 158, 128,   3, 153,  48,  99, 128, 216,  33, 196, 
  2,  28,  46,  96,  93,  25,  33, 114,  80, 247,  53,  31,   2,  17,   9, 172,  34, 162,  64,  31, 
 16, 160,  33, 210, 223, 172,  49,  45,  16,  15,  49, 241,  10,   2,  29,  45, 227,  15,  16, 160, 
 33,  17,  28,  93,  45, 237, 163,  33, 239, 114, 247, 192,  26, 218,  49,  44, 253,  93, 162,  33, 
239,   0, 240,   0, 173,  97,  96,  15,   0,   9, 215,  11,   1,   0,  31, 172,  58,  67,   0,   0, 
254,  48,  16,   1, 250, 192,  18, 209, 128,   3,  16,   1, 254,  48,  33,  16, 154, 194,  20,  38, 
218,  81,  10,   2,  29,  47, 213, 218,  33,  10,   2,  17,  17, 197, 210, 253,  80,   0, 214,  10, 
  1,   7,  42, 214,  10,  18, 231,  41, 192,  26, 113,  33, 188, 112, 199,  28,  25,   3,  67,  73, 
144,   0,  43,  49,  95, 218,  64,  48,   5, 195,  18, 202, 156,  17,  21,   0,  89, 195,  56,   1, 
 93, 112, 112,   1, 195,  18,   0,  12,  17,  43, 193,   0,  28,   1,  33,  43, 193, 133,  37,   0, 
 95, 208, 192,  52,   1,  95, 124,   1,  65,  44,  82,  83,  21,  33, 239,   2,  80,  21,  64, 213, 
 21,  49,  80,   5, 240,  16, 191, 235, 224, 176,  31,  75, 131,   0,  64,  80,   0, 193,   9,   3, 
131,  52, 153,  73, 144,   0, 131,  59,  48, 127, 217,  64,  48,   7, 195,  18,  19, 156,  31, 172, 
 52,  96,  59, 214,  14,   3, 189, 112, 112,   1, 195,  18,   0,  12,  28, 169, 193,   0, 118,  64, 
 64, 154, 195,  46,   0,  28,   1, 108, 169, 199,  14,  18, 220,  17,  54,  83, 179,  23,  51, 177, 
 25, 128,   0,  65,  36,  16,  82, 118,  52,   3,  75,  48, 115,   3,  50, 116,  15, 100,   3, 190, 
  0, 122, 194,  23,   8,  51, 190,  11,   3, 254,   0, 124,  94, 248,  83, 176,  23,  65, 133,  43, 
  0,  60,  32, 162,  40,  83, 176,   7,  17, 106, 195,  20,  18, 229,  59, 134,  80,  97, 136,  44, 
  3, 180,  14,  83, 176,   1, 192,  48,  50, 179,  59,  64,  46,  51,  59,  64, 117,  59,  51, 176, 
  1, 192,  40, 224, 176,  63,  48,  43, 197, 199,  22,  17, 128,  43, 184,  49, 116,   5,   1, 124, 
 25,   3,  19,   1,  76, 162, 232,   1,  68,  73, 144,  43,  49, 159, 218,  64,  48,   5, 195,  18, 
 19,  28,  17,  21,   0,  89, 237,   7,   0,  28,   3, 128,  25, 215,   7,   0,  28,  49,  32,   0, 
193,  18, 188,  17,  25,   0,  89, 215,  26,   1, 150, 158,   0,   0,   0,   0,   0,   0,   0,  10, 
194,  68,  30,   0,  81,  30, 197,  82, 144,   5,  17, 106, 194,  21,  32,   2, 157, 245,  41,  98, 
254, 204,  65,  22,   0,  90, 192,  18,  33, 204,  35,  25,  33, 229, 128,  25, 197,  81, 145,  20, 
204,   1,  24,  49, 148,   2, 197,  80,  80,  25, 192, 120, 129,  20, 114, 156,  49, 142,   6,  60, 
112, 236, 166, 101,   6, 120,  40,  39,   1, 145,  24,   0,  95, 210,  64, 227,  25,  64,  69,  25, 
112, 208,  10,  17,  71,  25, 194, 113,  12,  17, 163,  41,  64,  44,  85,   9, 129,  54,  50, 148, 
  4,  82, 159, 186, 194,  16,  64,  49, 159, 211, 197, 229, 251, 197,  16, 154, 195,  26, 141, 129, 
156, 113, 163,  25, 251, 197, 235, 196, 114, 226, 194,  17,  16,   2, 144,  25, 197,  81, 147,  25, 
  2, 157, 224,   5,  80, 147,  25,   0, 151,  46,  44,   1,  97,   0, 199,  14, 202,  99,   0,  64, 
 80,   0, 193,  57,   5,  83,  73, 147,   0,  72, 246, 156,  49,  97,   0, 199,  14, 202, 108, 179, 
131,  21, 253, 164,   3,   0,  92,  50, 239, 105, 195,  22,  19, 156, 112, 236, 169, 193,  17,  80, 
  5, 156,  49,  33,  43, 193,   0,  29,   0, 127, 105, 192,  22,  16,  12, 112, 236, 166, 216,  39, 
204,  17,  53,  82,  88,  50,  86,  80,  64,  24,   0,   4,  18,  65,   5,   0,  99,  61,  49,  95, 
218,  64, 240,  37,  81,  86,  64,  60,  16,   5, 172,  33, 112, 131,  37, 224, 176,  64,  16,   5, 
197, 239, 133,  37,   2,  81,  51, 172,  50,  99,  39,  33, 243,  50,  81,  51, 227,   2,  81,  58, 
172,  52, 141, 161,  33, 243, 246, 156,  49,  99,  39,  33, 243,  19,  48,  37, 197,  82,  80,  37, 
105,  25,  33, 251,  84,  68,  45,  24,  70, 172,  50, 205, 160,  33, 243,  19, 160,  37, 197,  82, 
 83,  37,  35, 229,   0,  81,  36, 172,  32, 225,  67,   5,  17, 158,  48,  37,  81,  88,  80,  83, 
  5,  49,  84,   2, 227, 130,  22,  80, 111,   8,  39, 131,  37,  50,  84,   1,  64, 227,  21,  64, 
 69,  21,  18, 192,  21, 198, 192,  16, 133,  21, 112, 214,  10,  34,   1,   9, 195, 255,   1, 140, 
  1,  85,  37,  98, 192,   0,  90, 194,  16,   8,  49,  80,   5, 223, 239, 133,  21,  18, 225,  46, 
131,  21,  49,  84,   1,  36, 210, 197,  64, 213,  21,  34,   1,   9, 194,  13, 218,  98, 254,  80, 
 94, 129,  66,  85,  13,  17,   5, 112, 194,  26,  24,  49,  85,  43,  50,  85,  47,   0,  93, 245, 
  5,  50,  85,  43,  49,  85,  47, 211,  47, 195,  18,  18, 188,  19,   5,   2, 180,   2, 227,  48, 
 80,  47,  64,  45, 232,   2, 244,   7,   2, 180,   4,  81,  87,  21,  39,  49,  81,  24,   0,  91, 
  2, 115,   0,  64,  80,   0, 193,  57,   5,  19,   1,  56, 232,   1,  67,  73, 144,  43,  49, 115, 
  7, 253, 148,   3,   0, 124,  50, 239, 105, 195,  22,  19, 140, 112, 236, 169, 193, 250, 195, 168, 
  1, 124,   1,  32,   0, 193,   0,  28,  34, 178, 194,  32,  16, 156,  51,  70, 155, 255, 146,  31, 
181,  68,  66, 209, 140, 112, 225,  43, 193,  34,   1,   9, 195,  22,  19, 140, 112, 225,  43, 193, 
 34,   1,   9, 192, 255,   2, 204,   0, 113,  22, 173,   2,  80,   7,  17, 153, 215,  26,   1, 118, 
155,  80,  79,  51,  47, 157, 230,  72,  74, 194, 101, 255,   1, 154,  99,  64,  50, 115,   7, 253, 
148,  15,  18, 232,  50, 115,  39,  64,  28,  85,  39,  50, 116,   7,  82, 120,   2, 116,  13,  82, 
120,  50, 115,  23,  64,  21,  23, 209,   0, 127, 105, 195,  22,  16,  12, 112, 236, 166, 204,  33, 
 22,   0, 122, 215,  22,   0, 113,  23, 157,  96, 177,  44,   1, 122, 192,  18,  18, 188,  16,   7, 
 17, 121, 194, 130,  20, 128,  23, 197,  82, 120, 216,   3, 196, 105,  96,   0,   0,   0,   0,   0, 
  0,   0,   2, 124, 110, 113,  34, 211,  39,  34, 229, 209,   0,  51,  23,  64, 117,  23, 136,   2, 
114,  29,  40,  50, 115,  39,  64,  17,  11,   0,  61,  92, 220,  84,  13,  49, 116,   4,  81, 126, 
 11,   4,   0,   0,  60,  85,   7, 133,  23,  17, 213,   7,   1,  12,  49,  97,   0, 199,  14, 202, 
108, 192,   1, 124,   1, 227,   7,  49, 116,   2, 227, 209, 128, 112,   7,  18,  74, 195,  82,  49, 
116,   7,  81, 126,  11,   7, 252,   0, 124,  83,  23,  64,  44,  85,   7,  33, 204,   1,  25,   0, 
121, 194, 184,  42, 136, 101,   7,  40,  39,  39,  18,  80,   7, 172,  49, 108, 169, 199,  56, 131, 
 23,  49, 116,   1,  34,  17, 176,   7, 197, 213, 205,  64, 227,  23,  64,  69,  23, 114, 192,  96, 
 19,  49,  38,   0, 122, 195,  22, 202, 156, 114,   0,  23,  17, 176,   7, 253,  44,  85,  23,  17, 
213,   7, 246, 157,  96, 112,   1, 192, 104,  19,  81,  38,   0, 122, 195,  22, 202, 156, 114,   0, 
 23,  17, 176,   7, 253,  44,  85,  23,  17, 213,   7, 133,   1, 105,  25,  33, 251,  84,  68,  45, 
 24,  71,  81, 115,  23,   0, 126,   6, 140,  80,   1,  48,   4,   5,   0,  12,  18,  60, 138, 151, 
 49,  16, 255,  11, 169, 108,  99, 115, 197, 203,  82, 143, 240, 228, 254,  12, 154, 146,  92, 154, 
156, 240,  15,   1, 250, 144,  79,   3, 250, 156, 194, 255,  26, 156, 240,  63, 169, 200,   3, 250, 
146,  28, 106, 156, 219, 169,  18, 162,  28, 157,  81,  41, 178,  28, 239, 230,  45, 254, 255, 230, 
 16, 140, 223,  63, 230,  37, 254, 241,  33, 254,  95, 238, 254, 223,  18, 156, 225,  32, 254,  79, 
238, 254,  65,   4, 241,  45, 206, 169, 201, 244, 191, 237, 210, 191, 228, 186, 159, 215, 196, 169, 
202,  42, 170, 145,   6, 205,  17,  92, 106, 158, 108,  74, 158, 188,  74, 145,   3, 205, 169,  16, 
 12, 106, 147,   5,  48,  26, 144,   9,   0,  90, 144,   5,   0, 154, 156, 221, 234, 156,  92, 218, 
144,   1,   0,  63, 219, 169,  33,   1,  10, 158, 204,  74, 147,  27,  48, 127, 217, 169,  50, 179, 
 11,  48,  51,  16, 169,  17,  80,   7, 154, 145,  21,   0, 185, 169, 252,  48,  10, 145,   0, 186, 
147,   1,  48,  10, 158,  16,   3, 233, 206, 169,  16,  27, 169, 216,   5, 196, 169,   0,  87, 107, 
169, 188, 154, 144,   5, 114, 228, 154, 146,  45, 186, 157,  81,   7, 169,  37, 244, 169, 210, 243, 
169,  41, 201, 169,  18, 140, 109,  90, 159, 229,  16,  74, 146, 220, 154, 144
};

uint8 get_code_nibble2(PROGRAM_COUNTER_TYPE pc)
{
  return (pc&1) ? ((uint8)(prgmem[pc>>1]&15)) : ((uint8)((prgmem[pc>>1]>>4)&15));
}

uint8 get_code_byte2(PROGRAM_COUNTER_TYPE pc)
{
	//return (get_code_nibble(pc)<<4)+get_code_nibble(pc+1);
	if(0==(pc&1))
		return prgmem[pc>>1];
	else
		return (((uint8)(prgmem[pc>>1]&15))<<4) + ((uint8)((prgmem[(pc+1)>>1]>>4)&15));
}

void main2()
{
	const uint16 stack_size=65535, memory_size=65535;
	static uint8 stack[stack_size], memory[memory_size];

	BytecodeInterpreter::BytecodeInterpreter ip(stack, stack_size, memory, memory_size, get_code_nibble2, get_code_byte2, &extern_function_invoker);
	ip.execute_bytecode(3335);
	while(_getch()==0);
}
